@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004-2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,29 +43,30 @@
 #include <wx/tooltip.h>
 
 #include <advanced_config.h>
-#include <cli/cli_names.h> // Needed for the pre wx 3.2 cli workaround
+#include <background_jobs_monitor.h>
+#include <bitmaps.h>
+#include <build_version.h>
 #include <common.h>
-#include <config_params.h>
 #include <confirm.h>
 #include <core/arraydim.h>
-#include <eda_base_frame.h>
-#include <eda_draw_frame.h>
-#include <gestfich.h>
 #include <id.h>
 #include <kicad_curl/kicad_curl.h>
 #include <kiplatform/policy.h>
 #include <macros.h>
-#include <menus_helpers.h>
+#include <notifications_manager.h>
 #include <paths.h>
 #include <pgm_base.h>
 #include <policy_keys.h>
 #include <python_scripting.h>
 #include <settings/common_settings.h>
 #include <settings/settings_manager.h>
+#include <string_utils.h>
 #include <systemdirsappend.h>
-#include <thread_pool.h>
+#include <core/thread_pool.h>
 #include <trace_helpers.h>
 
+#include <widgets/wx_splash.h>
+
 #ifdef KICAD_USE_SENTRY
 #include <boost/uuid/uuid_io.hpp>
 #include <boost/uuid/uuid_generators.hpp>
@@ -94,18 +95,20 @@ LANGUAGE_DESCR LanguagesList[] =
     { wxLANGUAGE_GREEK,      ID_LANGUAGE_GREEK,      wxT( "Ελληνικά" ), true },
     { wxLANGUAGE_ENGLISH,    ID_LANGUAGE_ENGLISH,    wxT( "English" ),  true },
     { wxLANGUAGE_SPANISH,    ID_LANGUAGE_SPANISH,    wxT( "Español" ),  true },
-    { wxLANGUAGE_SPANISH_MEXICAN, ID_LANGUAGE_SPANISH_MEXICAN, wxT( "Español (Latinoamericano)" ),  true },
+    { wxLANGUAGE_SPANISH_MEXICAN, ID_LANGUAGE_SPANISH_MEXICAN,
+      wxT( "Español (Latinoamericano)" ),  true },
     { wxLANGUAGE_FRENCH,     ID_LANGUAGE_FRENCH,     wxT( "Français" ), true },
     { wxLANGUAGE_KOREAN,     ID_LANGUAGE_KOREAN,     wxT( "한국어"),       true },
     { wxLANGUAGE_ITALIAN,    ID_LANGUAGE_ITALIAN,    wxT( "Italiano" ), true },
     { wxLANGUAGE_LITHUANIAN, ID_LANGUAGE_LITHUANIAN, wxT( "Lietuvių" ), true },
 //    { wxLANGUAGE_HUNGARIAN,  ID_LANGUAGE_HUNGARIAN,  wxT( "Magyar" ),   true },
-    { wxLANGUAGE_DUTCH,      ID_LANGUAGE_DUTCH,      wxT( "Nederlandse" ), true },
+    { wxLANGUAGE_DUTCH,      ID_LANGUAGE_DUTCH,      wxT( "Nederlands" ), true },
     { wxLANGUAGE_JAPANESE,   ID_LANGUAGE_JAPANESE,   wxT( "日本語" ),    true },
     { wxLANGUAGE_THAI,       ID_LANGUAGE_THAI,       wxT( "ภาษาไทย" ),    true },
     { wxLANGUAGE_POLISH,     ID_LANGUAGE_POLISH,     wxT( "Polski" ),   true },
     { wxLANGUAGE_PORTUGUESE, ID_LANGUAGE_PORTUGUESE, wxT( "Português" ),true },
-    { wxLANGUAGE_PORTUGUESE_BRAZILIAN, ID_LANGUAGE_PORTUGUESE_BRAZILIAN, wxT( "Português (Brasil)" ), true },
+    { wxLANGUAGE_PORTUGUESE_BRAZILIAN, ID_LANGUAGE_PORTUGUESE_BRAZILIAN,
+      wxT( "Português (Brasil)" ), true },
     { wxLANGUAGE_RUSSIAN,    ID_LANGUAGE_RUSSIAN,    wxT( "Русский" ),  true },
 //    { wxLANGUAGE_SERBIAN,    ID_LANGUAGE_SERBIAN,    wxT( "Српски" ),   true },
     { wxLANGUAGE_FINNISH,    ID_LANGUAGE_FINNISH,    wxT( "Suomi" ),    true },
@@ -116,7 +119,7 @@ LANGUAGE_DESCR LanguagesList[] =
     { wxLANGUAGE_CHINESE_SIMPLIFIED, ID_LANGUAGE_CHINESE_SIMPLIFIED,
             wxT( "简体中文" ), true },
     { wxLANGUAGE_CHINESE_TRADITIONAL, ID_LANGUAGE_CHINESE_TRADITIONAL,
-            wxT( "繁體中文" ), false },
+            wxT( "繁體中文" ), true },
     { 0, 0, "", false }         // Sentinel
 };
 #undef _
@@ -130,6 +133,7 @@ PGM_BASE::PGM_BASE()
     m_Quitting = false;
     m_argcUtf8 = 0;
     m_argvUtf8 = nullptr;
+    m_splash = nullptr;
 
     setLanguageId( wxLANGUAGE_DEFAULT );
 
@@ -139,6 +143,7 @@ PGM_BASE::PGM_BASE()
 
 PGM_BASE::~PGM_BASE()
 {
+    HideSplash();
     Destroy();
 
     for( int n = 0; n < m_argcUtf8; n++ )
@@ -321,20 +326,30 @@ void PGM_BASE::sentryInit()
 
         sentry_options_t* options = sentry_options_new();
 
-        sentry_options_set_dsn(
-                options,
-                "https://463925e689c34632b5172436ffb76de5@sentry-relay.kicad.org/6266565" );
+        #ifndef KICAD_SENTRY_DSN
+        #   error "Project configuration error, missing KICAD_SENTRY_DSN"
+        #endif
+
+        sentry_options_set_dsn( options, KICAD_SENTRY_DSN );
 
         wxFileName tmp;
         tmp.AssignDir( PATHS::GetUserCachePath() );
         tmp.AppendDir( "sentry" );
 
+#ifdef __WINDOWS__
         sentry_options_set_database_pathw( options, tmp.GetPathWithSep().wc_str() );
+#else
+        sentry_options_set_database_path( options, tmp.GetPathWithSep().c_str() );
+#endif
         sentry_options_set_symbolize_stacktraces( options, true );
         sentry_options_set_auto_session_tracking( options, false );
 
         sentry_options_set_release( options, GetCommitHash().ToStdString().c_str() );
 
+        // This just gives us more filtering within sentry, issues still get grouped across
+        // environments.
+        sentry_options_set_environment( options, GetMajorMinorVersion().c_str() );
+
         sentry_init( options );
 
         sentry_value_t user = sentry_value_new_object();
@@ -401,6 +416,36 @@ void PGM_BASE::BuildArgvUtf8()
 }
 
 
+void PGM_BASE::ShowSplash()
+{
+    // Disabling until we change to load each DSO at startup rather than lazy-load when needed.
+    // Note that once the splash screen is re-enabled, there are some remaining bugs to fix:
+    // Any wxWidgets error dialogs that appear during startup are hidden by the splash screen,
+    // so we either need to prevent these from happening (probably not feasible) or else change
+    // the error-handling path to make sure errors go on top of the splash.
+#if 0
+    if( m_splash )
+        return;
+
+    m_splash = new WX_SPLASH( KiBitmap( BITMAPS::splash ), wxSPLASH_CENTRE_ON_SCREEN, 0,
+                              NULL, -1, wxDefaultPosition, wxDefaultSize,
+                              wxBORDER_NONE | wxSTAY_ON_TOP );
+    wxYield();
+#endif
+}
+
+
+void PGM_BASE::HideSplash()
+{
+    if( !m_splash )
+        return;
+
+    m_splash->Close( true );
+    m_splash->Destroy();
+    m_splash = nullptr;
+}
+
+
 bool PGM_BASE::InitPgm( bool aHeadless, bool aSkipPyInit, bool aIsUnitTest )
 {
 #if defined( __WXMAC__ )
@@ -445,8 +490,15 @@ bool PGM_BASE::InitPgm( bool aHeadless, bool aSkipPyInit, bool aIsUnitTest )
         return false;
     }
 #endif
+
+    wxString instanceCheckerDir = PATHS::GetInstanceCheckerPath();
+    PATHS::EnsurePathExists( instanceCheckerDir );
+
+    wxString instanceCheckerName = wxString::Format( wxS( "%s-%s" ), pgm_name,
+                                                     GetMajorMinorVersion() );
+
     m_pgm_checker = std::make_unique<wxSingleInstanceChecker>();
-    m_pgm_checker->Create( pgm_name, wxStandardPaths::Get().GetTempDir() );
+    m_pgm_checker->Create( instanceCheckerName, instanceCheckerDir );
 
     // Init KiCad environment
     // the environment variable KICAD (if exists) gives the kicad path:
@@ -479,7 +531,6 @@ bool PGM_BASE::InitPgm( bool aHeadless, bool aSkipPyInit, bool aIsUnitTest )
 
     // Analyze the command line & initialize the binary path
     wxString tmp;
-    setExecutablePath();
     SetLanguagePath();
     SetDefaultLanguage( tmp );
 
@@ -491,6 +542,8 @@ bool PGM_BASE::InitPgm( bool aHeadless, bool aSkipPyInit, bool aIsUnitTest )
 #endif
 
     m_settings_manager = std::make_unique<SETTINGS_MANAGER>( aHeadless );
+    m_background_jobs_monitor = std::make_unique<BACKGROUND_JOBS_MONITOR>();
+    m_notifications_manager = std::make_unique<NOTIFICATIONS_MANAGER>();
 
     // Our unit test mocks break if we continue
     // A bug caused InitPgm to terminate early in unit tests and the mocks are...simplistic
@@ -527,6 +580,8 @@ bool PGM_BASE::InitPgm( bool aHeadless, bool aSkipPyInit, bool aIsUnitTest )
 
     ReadPdfBrowserInfos();      // needs GetCommonSettings()
 
+    GetNotificationsManager().Load();
+
     // Create the python scripting stuff
     // Skip it fot applications that do not use it
     if( !aSkipPyInit )
@@ -547,47 +602,9 @@ bool PGM_BASE::InitPgm( bool aHeadless, bool aSkipPyInit, bool aIsUnitTest )
     if( ADVANCED_CFG::GetCfg().m_UpdateUIEventInterval != 0 )
         wxUpdateUIEvent::SetUpdateInterval( ADVANCED_CFG::GetCfg().m_UpdateUIEventInterval );
 
-    return true;
-}
-
-
-bool PGM_BASE::setExecutablePath()
-{
-    m_bin_dir = wxStandardPaths::Get().GetExecutablePath();
-
-#ifdef __WXMAC__
-    // On OSX Pgm().GetExecutablePath() will always point to main
-    // bundle directory, e.g., /Applications/kicad.app/
-
-    wxFileName fn( m_bin_dir );
-
-    if( fn.GetName() == wxT( "kicad" ) || fn.GetName() == wxT( "kicad-cli" ) )
-    {
-        // kicad launcher, so just remove the Contents/MacOS part
-        fn.RemoveLastDir();
-        fn.RemoveLastDir();
-    }
-    else
-    {
-        // standalone binaries live in Contents/Applications/<standalone>.app/Contents/MacOS
-        fn.RemoveLastDir();
-        fn.RemoveLastDir();
-        fn.RemoveLastDir();
-        fn.RemoveLastDir();
-        fn.RemoveLastDir();
-    }
-
-    m_bin_dir = fn.GetPath() + wxT( "/" );
-#else
-    // Use unix notation for paths. I am not sure this is a good idea,
-    // but it simplifies compatibility between Windows and Unices.
-    // However it is a potential problem in path handling under Windows.
-    m_bin_dir.Replace( WIN_STRING_DIR_SEP, UNIX_STRING_DIR_SEP );
-
-    // Remove file name form command line:
-    while( m_bin_dir.Last() != '/' && !m_bin_dir.IsEmpty() )
-        m_bin_dir.RemoveLast();
-#endif
+    // Now the application can safely start, show the splash screen
+    if( !aHeadless )
+        ShowSplash();
 
     return true;
 }
@@ -643,6 +660,7 @@ bool PGM_BASE::SetLanguage( wxString& aErrMsg, bool first_time )
     if( first_time )
     {
         setLanguageId( wxLANGUAGE_DEFAULT );
+
         // First time SetLanguage is called, the user selected language id is set
         // from common user config settings
         wxString languageSel = GetCommonSettings()->m_System.language;
@@ -800,11 +818,7 @@ wxString PGM_BASE::GetLanguageTag()
         return "";
     else
     {
-#if wxCHECK_VERSION( 3, 1, 6 )
         wxString str = langInfo->GetCanonicalWithRegion();
-#else
-        wxString str = langInfo->CanonicalName;
-#endif
         str.Replace( "_", "-" );
 
         return str;
@@ -814,6 +828,10 @@ wxString PGM_BASE::GetLanguageTag()
 
 void PGM_BASE::SetLanguagePath()
 {
+#ifdef _MSC_VER
+    wxLocale::AddCatalogLookupPathPrefix( PATHS::GetWindowsBaseSharePath()
+                                                  + wxT( "locale" ) );
+#endif
     wxLocale::AddCatalogLookupPathPrefix( PATHS::GetLocaleDataPath() );
 
     if( wxGetEnv( wxT( "KICAD_RUN_FROM_BUILD_DIR" ), nullptr ) )
@@ -833,7 +851,8 @@ bool PGM_BASE::SetLocalEnvVariable( const wxString& aName, const wxString& aValu
     if( aName.IsEmpty() )
     {
         wxLogTrace( traceEnvVars,
-                    wxT( "PGM_BASE::SetLocalEnvVariable: Attempt to set empty variable to value %s" ),
+                    wxT( "PGM_BASE::SetLocalEnvVariable: Attempt to set empty variable to "
+                         "value %s" ),
                     aValue );
         return false;
     }
@@ -842,7 +861,8 @@ bool PGM_BASE::SetLocalEnvVariable( const wxString& aName, const wxString& aValu
     if( wxGetEnv( aName, &env ) )
     {
         wxLogTrace( traceEnvVars,
-                    wxT( "PGM_BASE::SetLocalEnvVariable: Environment variable %s already set to %s" ),
+                    wxT( "PGM_BASE::SetLocalEnvVariable: Environment variable %s already set "
+                         "to %s" ),
                     aName, env );
         return env == aValue;
     }
@@ -862,7 +882,8 @@ void PGM_BASE::SetLocalEnvVariables()
     for( const std::pair<wxString, ENV_VAR_ITEM> m_local_env_var : GetCommonSettings()->m_Env.vars )
     {
         wxLogTrace( traceEnvVars,
-                    wxT( "PGM_BASE::SetLocalEnvVariables: Setting local environment variable %s to %s" ),
+                    wxT( "PGM_BASE::SetLocalEnvVariables: Setting local environment variable %s "
+                         "to %s" ),
                     m_local_env_var.first,
                     m_local_env_var.second.GetValue() );
         wxSetEnv( m_local_env_var.first, m_local_env_var.second.GetValue() );
@@ -881,16 +902,7 @@ bool PGM_BASE::IsGUI()
     if( !wxTheApp )
         return false;
 
-#if wxCHECK_VERSION( 3, 1, 6 )
     return wxTheApp->IsGUI();
-#else
-    // wxWidgets older than version 3.1.6 do not have a way to know if the app
-    // has a GUI or is a console application.
-    // So the trick is to set the App class name when starting kicad-cli, and when
-    // the app class name is the kicad-cli class name the app is a console app
-    bool run_gui = wxTheApp->GetClassName() != KICAD_CLI_APP_NAME;
-    return run_gui;
-#endif
 }
 
 
@@ -908,19 +920,107 @@ void PGM_BASE::HandleException( std::exception_ptr aPtr )
     catch( const std::exception& e )
     {
 #ifdef KICAD_USE_SENTRY
-        sentry_value_t exc = sentry_value_new_exception( "exception", e.what() );
-        sentry_value_set_stacktrace( exc, NULL, 0 );
+        if( IsSentryOptedIn() )
+        {
+            sentry_value_t exc = sentry_value_new_exception( "exception", e.what() );
+            sentry_value_set_stacktrace( exc, NULL, 0 );
 
-        sentry_value_t sentryEvent = sentry_value_new_event();
-        sentry_event_add_exception( sentryEvent, exc );
-        sentry_capture_event( sentryEvent );
+            sentry_value_t sentryEvent = sentry_value_new_event();
+            sentry_event_add_exception( sentryEvent, exc );
+            sentry_capture_event( sentryEvent );
+        }
 #endif
 
         wxLogError( wxT( "Unhandled exception class: %s  what: %s" ),
-                    FROM_UTF8( typeid( e ).name() ), FROM_UTF8( e.what() ) );
+                    From_UTF8( typeid( e ).name() ), From_UTF8( e.what() ) );
     }
     catch( ... )
     {
         wxLogError( wxT( "Unhandled exception of unknown type" ) );
     }
 }
+
+
+#ifdef KICAD_USE_SENTRY
+struct SENTRY_ASSERT_CACHE_KEY
+{
+    wxString file;
+    int      line;
+    wxString func;
+    wxString cond;
+    wxString msg;
+};
+
+
+bool operator<( const SENTRY_ASSERT_CACHE_KEY& aKey1, const SENTRY_ASSERT_CACHE_KEY& aKey2 )
+{
+    return aKey1.file < aKey2.file ||
+        aKey1.line < aKey2.line ||
+        aKey1.func < aKey2.func ||
+        aKey1.cond < aKey2.cond ||
+        aKey1.msg < aKey2.msg;
+}
+#endif
+
+
+void PGM_BASE::HandleAssert( const wxString& aFile, int aLine, const wxString& aFunc,
+                             const wxString& aCond, const wxString& aMsg )
+{
+    wxString assertStr;
+
+    // Log the assertion details to standard log
+    if( !aMsg.empty() )
+    {
+        assertStr = wxString::Format( "Assertion failed at %s:%d in %s: %s - %s", aFile, aLine,
+                                      aFunc, aCond, aMsg );
+    }
+    else
+    {
+        assertStr = wxString::Format( "Assertion failed at %s:%d in %s: %s", aFile, aLine, aFunc,
+                                      aCond );
+    }
+
+#ifndef NDEBUG
+    wxLogError( assertStr );
+#endif
+
+#ifdef KICAD_USE_SENTRY
+    if( IsSentryOptedIn() )
+    {
+        static std::set<SENTRY_ASSERT_CACHE_KEY> assertCache;
+
+        SENTRY_ASSERT_CACHE_KEY key = { aFile, aLine, aFunc, aCond, aMsg };
+
+        if( assertCache.find( key ) == assertCache.end() )
+        {
+            sentry_value_t exc = sentry_value_new_exception( "assert", assertStr );
+            sentry_value_set_stacktrace( exc, NULL, 0 );
+
+            sentry_value_t sentryEvent = sentry_value_new_event();
+            sentry_event_add_exception( sentryEvent, exc );
+            sentry_capture_event( sentryEvent );
+            assertCache.insert( key );
+        }
+    }
+#endif
+}
+
+
+const wxString& PGM_BASE::GetExecutablePath() const
+{
+    return PATHS::GetExecutablePath();
+}
+
+
+void PGM_BASE::ReadPdfBrowserInfos()
+{
+    SetPdfBrowserName( GetCommonSettings()->m_System.pdf_viewer_name );
+    m_use_system_pdf_browser = GetCommonSettings()->m_System.use_system_pdf_viewer;
+}
+
+
+void PGM_BASE::WritePdfBrowserInfos()
+{
+    GetCommonSettings()->m_System.pdf_viewer_name = GetPdfBrowserName();
+    GetCommonSettings()->m_System.use_system_pdf_viewer = m_use_system_pdf_browser;
+}
