@@ -80,6 +80,16 @@ public:
     void MirrorVertically( int aCenter ) override {}
     void Rotate( const VECTOR2I& aCenter ) override {}
 
+    double Similarity( const SCH_ITEM& aOther ) const override
+    {
+        return 0.0;
+    }
+
+    bool operator==( const SCH_ITEM& aOther ) const override
+    {
+        return false;
+    }
+
 #if defined(DEBUG)
     void Show( int , std::ostream&  ) const override {}
 #endif
@@ -464,7 +474,6 @@ bool SCH_SHEET_PATH::TestForRecursion( const wxString& aSrcFileName, const wxStr
     if( destFn.IsRelative() )
         destFn.MakeAbsolute( rootFn.GetPath() );
 
-
     // The source and destination sheet file names cannot be the same.
     if( srcFn == destFn )
     {
@@ -575,10 +584,10 @@ void SCH_SHEET_PATH::AddNewSymbolInstances( const SCH_SHEET_PATH& aPrefixSheetPa
             newSymbolInstance.m_Path = newSheetPath.Path();
             symbol->AddHierarchicalReference( newSymbolInstance );
         }
-        else if( !symbol->GetInstanceReferences().empty() )
+        else if( !symbol->GetInstances().empty() )
         {
             // Use the first symbol instance if any symbol instance data exists.
-            newSymbolInstance = symbol->GetInstanceReferences()[0];
+            newSymbolInstance = symbol->GetInstances()[0];
             newSymbolInstance.m_Path = newSheetPath.Path();
             symbol->AddHierarchicalReference( newSymbolInstance );
         }
@@ -772,6 +781,20 @@ bool SCH_SHEET_LIST::PageNumberExists( const wxString& aPageNumber ) const
 }
 
 
+void SCH_SHEET_LIST::TrimToPageNumbers( const std::vector<wxString>& aPageInclusions )
+{
+    auto it = std::remove_if( begin(), end(),
+                              [&]( SCH_SHEET_PATH sheet )
+                              {
+                                  return std::find( aPageInclusions.begin(), aPageInclusions.end(),
+                                                    sheet.GetPageNumber() )
+                                         == aPageInclusions.end();
+                              } );
+
+    erase( it, end() );
+}
+
+
 bool SCH_SHEET_LIST::IsModified() const
 {
     for( const SCH_SHEET_PATH& sheet : *this )
@@ -1143,7 +1166,7 @@ void SCH_SHEET_LIST::UpdateSheetInstanceData( const std::vector<SCH_SHEET_INSTAN
         }
 
         wxLogTrace( traceSchSheetPaths, "Setting sheet '%s' instance '%s' page number '%s'",
-                    ( sheet->GetName().IsEmpty() ) ? wxT( "root" ) : sheet->GetName(),
+                    ( sheet->GetName().IsEmpty() ) ? wxString( wxT( "root" ) ) : sheet->GetName(),
                     path.Path().AsString(), it->m_PageNumber );
         path.SetPageNumber( it->m_PageNumber );
     }
@@ -1270,13 +1293,15 @@ void SCH_SHEET_LIST::AddNewSheetInstances( const SCH_SHEET_PATH& aPrefixSheetPat
 
         SCH_SHEET_INSTANCE instance;
 
-        if( sheet->getInstance( instance, tmp.Path(), true )
-          && !instance.m_PageNumber.IsEmpty() )
+        // Add the instance if it doesn't already exist
+        if( !sheet->getInstance( instance, tmp.Path(), true ) )
         {
-            newSheetPath.SetPageNumber( instance.m_PageNumber );
-            usedPageNumbers.push_back( instance.m_PageNumber );
+            sheet->addInstance( tmp );
+            sheet->getInstance( instance, tmp.Path(), true );
         }
-        else
+
+        // Get a new page number if we don't have one
+        if( instance.m_PageNumber.IsEmpty() )
         {
             // Generate the next available page number.
             do
@@ -1286,10 +1311,12 @@ void SCH_SHEET_LIST::AddNewSheetInstances( const SCH_SHEET_PATH& aPrefixSheetPat
             } while( std::find( usedPageNumbers.begin(), usedPageNumbers.end(), pageNumber ) !=
                      usedPageNumbers.end() );
 
+            instance.m_PageNumber = pageNumber;
             newSheetPath.SetVirtualPageNumber( nextVirtualPageNumber );
-            newSheetPath.SetPageNumber( pageNumber );
-            usedPageNumbers.push_back( pageNumber );
         }
+
+        newSheetPath.SetPageNumber( instance.m_PageNumber );
+        usedPageNumbers.push_back( instance.m_PageNumber );
     }
 }
 
@@ -1318,3 +1345,18 @@ bool SCH_SHEET_LIST::HasPath( const KIID_PATH& aPath ) const
 
     return false;
 }
+
+
+bool SCH_SHEET_LIST::ContainsSheet( const SCH_SHEET* aSheet ) const
+{
+    for( const SCH_SHEET_PATH& path : *this )
+    {
+        for( size_t i = 0; i < path.size(); i++ )
+        {
+            if( path.at( i ) == aSheet )
+                return true;
+        }
+    }
+
+    return false;
+}
