@@ -23,12 +23,15 @@
  */
 
 #include <cmath>
+#include <wx/log.h>
 #include <trigo.h>
+#include <gal/graphics_abstraction_layer.h>
 #include <tool/tool_manager.h>
 #include <tools/ee_grid_helper.h>
 #include <tools/ee_selection_tool.h>
 #include <tools/sch_line_wire_bus_tool.h>
 #include <ee_actions.h>
+#include <sch_commit.h>
 #include <eda_item.h>
 #include <sch_item.h>
 #include <sch_symbol.h>
@@ -85,8 +88,8 @@ bool SCH_MOVE_TOOL::Init()
 }
 
 
-void SCH_MOVE_TOOL::orthoLineDrag( SCH_LINE* line, const VECTOR2I& splitDelta, int& xBendCount,
-                                   int& yBendCount, const EE_GRID_HELPER& grid )
+void SCH_MOVE_TOOL::orthoLineDrag( SCH_COMMIT* aCommit, SCH_LINE* line, const VECTOR2I& splitDelta,
+                                   int& xBendCount, int& yBendCount, const EE_GRID_HELPER& grid )
 {
     // If the move is not the same angle as this move,  then we need to do something special with
     // the unselected end to maintain orthogonality. Either drag some connected line that is the
@@ -195,7 +198,7 @@ void SCH_MOVE_TOOL::orthoLineDrag( SCH_LINE* line, const VECTOR2I& splitDelta, i
 
             if( !foundLine->HasFlag( IS_CHANGED ) && !foundLine->HasFlag( IS_NEW ) )
             {
-                saveCopyInUndoList( (SCH_ITEM*) foundLine, UNDO_REDO::CHANGED, true );
+                aCommit->Modify( (SCH_ITEM*) foundLine, m_frame->GetScreen() );
 
                 if( !foundLine->IsSelected() )
                     m_changedDragLines.insert( foundLine );
@@ -277,6 +280,8 @@ void SCH_MOVE_TOOL::orthoLineDrag( SCH_LINE* line, const VECTOR2I& splitDelta, i
         // put wires between it and its original endpoint.
         else if( foundAttachment && line->IsOrthogonal() )
         {
+            VECTOR2D lineGrid = grid.GetGridSize( grid.GetItemGrid( line ) );
+
             // The bend counter handles a group of wires all needing their offset one grid movement
             // further out from each other to not overlap.  The absolute value stuff finds the
             // direction of the line and hence the the bend increment on that axis
@@ -284,9 +289,9 @@ void SCH_MOVE_TOOL::orthoLineDrag( SCH_LINE* line, const VECTOR2I& splitDelta, i
             unsigned int yMoveBit = splitDelta.y != 0;
             int          xLength = abs( unselectedEnd.x - selectedEnd.x );
             int          yLength = abs( unselectedEnd.y - selectedEnd.y );
-            int          xMove = ( xLength - ( xBendCount * grid.GetGrid().x ) )
+            int          xMove = ( xLength - ( xBendCount * lineGrid.x ) )
                                     * sign( selectedEnd.x - unselectedEnd.x );
-            int          yMove = ( yLength - ( yBendCount * grid.GetGrid().y ) )
+            int          yMove = ( yLength - ( yBendCount * lineGrid.y ) )
                                     * sign( selectedEnd.y - unselectedEnd.y );
 
             // Create a new wire ending at the unselected end, we'll move the new wire's start
@@ -353,32 +358,56 @@ void SCH_MOVE_TOOL::orthoLineDrag( SCH_LINE* line, const VECTOR2I& splitDelta, i
 
 int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
 {
-    EESCHEMA_SETTINGS*    cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
-    KIGFX::VIEW_CONTROLS* controls = getViewControls();
-    EE_GRID_HELPER        grid( m_toolMgr );
-    bool                  wasDragging = m_moveInProgress && m_isDrag;
-    bool                  isSlice = false;
+    m_isDrag = aEvent.IsAction( &EE_ACTIONS::drag );
 
-    m_anchorPos.reset();
+    if( SCH_COMMIT* commit = dynamic_cast<SCH_COMMIT*>( aEvent.Commit() ) )
+    {
+        bool isSlice = false;
 
-    if( aEvent.IsAction( &EE_ACTIONS::move ) )
-    {
-        m_isDrag = false;
-    }
-    else if( aEvent.IsAction( &EE_ACTIONS::drag ) )
-    {
-        m_isDrag = true;
-        isSlice = aEvent.Parameter<bool>();
-    }
-    else if( aEvent.IsAction( &EE_ACTIONS::moveActivate ) )
-    {
-        m_isDrag = !cfg->m_Input.drag_is_move;
+        if( m_isDrag )
+            isSlice = aEvent.Parameter<bool>();
+
+        wxCHECK( aEvent.SynchronousState(), 0 );
+        aEvent.SynchronousState()->store( STS_RUNNING );
+
+        if( doMoveSelection( aEvent, commit, isSlice ) )
+            aEvent.SynchronousState()->store( STS_FINISHED );
+        else
+            aEvent.SynchronousState()->store( STS_CANCELLED );
     }
     else
     {
-        return 0;
+        SCH_COMMIT localCommit( m_toolMgr );
+
+        if( doMoveSelection( aEvent, &localCommit, false ) )
+            localCommit.Push( m_isDrag ? _( "Drag" ) : _( "Move" ) );
+        else
+            localCommit.Revert();
     }
 
+    return 0;
+}
+
+
+bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aCommit, bool aIsSlice )
+{
+    EESCHEMA_SETTINGS*    cfg = nullptr;
+
+    try
+    {
+        cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
+    }
+    catch( const std::runtime_error& e )
+    {
+        wxCHECK_MSG( false, false, e.what() );
+    }
+
+    KIGFX::VIEW_CONTROLS* controls = getViewControls();
+    EE_GRID_HELPER        grid( m_toolMgr );
+    bool                  wasDragging = m_moveInProgress && m_isDrag;
+
+    m_anchorPos.reset();
+
     if( m_moveInProgress )
     {
         if( m_isDrag != wasDragging )
@@ -389,16 +418,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
             {
                 // Reset the selected items so we can start again with the current m_isDrag
                 // state.
-                try
-                {
-                    m_frame->RollbackSchematicFromUndo();
-                }
-                catch( const IO_ERROR& exc )
-                {
-                    wxLogWarning( wxS( "Exception \"%s\" rolling back schematic undo ocurred." ),
-                                  exc.What() );
-                    return 1;
-                }
+                aCommit->Revert();
 
                 m_selectionTool->RemoveItemsFromSel( &m_dragAdditions, QUIET_MODE );
                 m_anchorPos = m_cursor - m_moveOffset;
@@ -407,20 +427,20 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
 
                 // And give it a kick so it doesn't have to wait for the first mouse movement
                 // to refresh.
-                m_toolMgr->RunAction( EE_ACTIONS::restartMove );
+                m_toolMgr->PostAction( EE_ACTIONS::restartMove );
             }
         }
         else
         {
             // The tool hotkey is interpreted as a click when already dragging/moving
-            m_toolMgr->RunAction( ACTIONS::cursorClick );
+            m_toolMgr->PostAction( ACTIONS::cursorClick );
         }
 
-        return 0;
+        return false;
     }
 
     if( m_inMoveTool )      // Must come after m_moveInProgress checks above...
-        return 0;
+        return false;
 
     REENTRANCY_GUARD guard( &m_inMoveTool );
 
@@ -444,15 +464,14 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
         // Note that it's important to go through push/pop even when the selection is empty.
         // This keeps other tools from having to special-case an empty move.
         m_frame->PopTool( aEvent );
-        return 0;
+        return false;
     }
 
     bool        restore_state = false;
-    bool        chain_commands = false;
     TOOL_EVENT  copy = aEvent;
     TOOL_EVENT* evt = &copy;
     VECTOR2I    prevPos;
-    int         snapLayer = UNDEFINED_LAYER;
+    GRID_HELPER_GRIDS snapLayer = GRID_CURRENT;
 
     m_cursor = controls->GetCursorPosition();
 
@@ -463,8 +482,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
         grid.SetSnap( !evt->Modifier( MD_SHIFT ) );
         grid.SetUseGrid( getView()->GetGAL()->GetGridSnapping() && !evt->DisableGridSnapping() );
 
-        if( evt->IsAction( &EE_ACTIONS::moveActivate )
-                || evt->IsAction( &EE_ACTIONS::restartMove )
+        if( evt->IsAction( &EE_ACTIONS::restartMove )
                 || evt->IsAction( &EE_ACTIONS::move )
                 || evt->IsAction( &EE_ACTIONS::drag )
                 || evt->IsMotion()
@@ -474,7 +492,6 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
             if( !m_moveInProgress )    // Prepare to start moving/dragging
             {
                 SCH_ITEM* sch_item = (SCH_ITEM*) selection.Front();
-                bool      appendUndo = ( sch_item && sch_item->IsNew() ) || isSlice;
                 bool      placingNewItems = sch_item && sch_item->IsNew();
 
                 //------------------------------------------------------------------------
@@ -496,7 +513,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
 
                 // Drag of split items start over top of their other segment so
                 // we want to skip grabbing the segments we split from
-                if( m_isDrag && !isSlice )
+                if( m_isDrag && !aIsSlice )
                 {
                     EDA_ITEMS connectedDragItems;
 
@@ -527,7 +544,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
                         }
 
                         for( const VECTOR2I& point : connections )
-                            getConnectedDragItems( item, point, connectedDragItems, appendUndo );
+                            getConnectedDragItems( aCommit, item, point, connectedDragItems );
                     }
 
                     // Go back and get all label connections now that we can test for drag-selected
@@ -535,7 +552,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
                     for( SCH_ITEM* item : stageTwo )
                     {
                         for( const VECTOR2I& point : item->GetConnectionPoints() )
-                            getConnectedDragItems( item, point, connectedDragItems, appendUndo );
+                            getConnectedDragItems( aCommit, item, point, connectedDragItems );
                     }
 
                     for( EDA_ITEM* item : connectedDragItems )
@@ -573,42 +590,29 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
                         static_cast<SCH_ITEM*>( item )->UpdateDanglingState( internalPoints );
                 }
 
+
                 // Generic setup
-                //
+                snapLayer = grid.GetSelectionGrid( selection );
+
                 for( EDA_ITEM* item : selection )
                 {
-                    if( static_cast<SCH_ITEM*>( item )->IsConnectable() )
-                    {
-                        if( snapLayer == LAYER_GRAPHICS )
-                            snapLayer = LAYER_ANY;
-                        else
-                            snapLayer = LAYER_CONNECTABLE;
-                    }
-                    else
-                    {
-                        if( snapLayer == LAYER_CONNECTABLE )
-                            snapLayer = LAYER_ANY;
-                        else
-                            snapLayer = LAYER_GRAPHICS;
-                    }
-
                     if( item->IsNew() )
                     {
-                        // Item was added in a previous command (and saved to undo by
-                        // that command)
+                        // Item was added to commit in a previous command
                     }
                     else if( item->GetParent() && item->GetParent()->IsSelected() )
                     {
-                        // Item will be (or has been) saved to undo by parent
+                        // Item will be (or has been) added to commit by parent
                     }
                     else
                     {
-                        saveCopyInUndoList( (SCH_ITEM*) item, UNDO_REDO::CHANGED, appendUndo );
-                        appendUndo = true;
+                        aCommit->Modify( item, m_frame->GetScreen() );
                     }
 
-                    SCH_ITEM* schItem = (SCH_ITEM*) item;
-                    schItem->SetStoredPos( schItem->GetPosition() );
+                    item->SetFlags( IS_MOVING );
+
+                    if( SCH_ITEM* schItem = dynamic_cast<SCH_ITEM*>( item ) )
+                        schItem->SetStoredPos( schItem->GetPosition() );
                 }
 
                 // Set up the starting position and move/drag offset
@@ -674,7 +678,6 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
                 }
 
                 controls->SetCursorPosition( m_cursor, false );
-                m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
 
                 prevPos = m_cursor;
                 controls->SetAutoPan( true );
@@ -748,7 +751,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
                             && line
                             && line->HasFlag( STARTPOINT ) != line->HasFlag( ENDPOINT ) )
                     {
-                        orthoLineDrag( line, splitDelta, xBendCount, yBendCount, grid );
+                        orthoLineDrag( aCommit, line, splitDelta, xBendCount, yBendCount, grid );
                     }
 
                     // Move all other items normally, including the selected end of partially
@@ -823,40 +826,28 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
         }
         else if( evt->IsAction( &ACTIONS::duplicate ) )
         {
-            if( selection.Front() && selection.Front()->IsNew() )
-            {
-                // This doesn't really make sense; we'll just end up dragging a stack of
-                // objects so we ignore the duplicate and just carry on.
-                wxBell();
-                continue;
-            }
-
-            // Move original back and exit.  The duplicate will run in its own loop.
-            restore_state = true;
-            unselect = false;
-            chain_commands = true;
-            break;
+            wxBell();
         }
         else if( evt->IsAction( &EE_ACTIONS::rotateCW ) )
         {
-            m_toolMgr->RunAction( EE_ACTIONS::rotateCW, true );
+            m_toolMgr->RunSynchronousAction( EE_ACTIONS::rotateCW, aCommit );
         }
         else if( evt->IsAction( &EE_ACTIONS::rotateCCW ) )
         {
-            m_toolMgr->RunAction( EE_ACTIONS::rotateCCW, true );
+            m_toolMgr->RunSynchronousAction( EE_ACTIONS::rotateCCW, aCommit );
         }
         else if( evt->Action() == TA_CHOICE_MENU_CHOICE )
         {
-            if( *evt->GetCommandId() >= ID_POPUP_SCH_SELECT_UNIT_CMP
-                && *evt->GetCommandId() <= ID_POPUP_SCH_SELECT_UNIT_SYM_MAX )
+            if( *evt->GetCommandId() >= ID_POPUP_SCH_SELECT_UNIT
+                && *evt->GetCommandId() <= ID_POPUP_SCH_SELECT_UNIT_END )
             {
                 SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( selection.Front() );
-                int unit = *evt->GetCommandId() - ID_POPUP_SCH_SELECT_UNIT_CMP;
+                int unit = *evt->GetCommandId() - ID_POPUP_SCH_SELECT_UNIT;
 
                 if( symbol )
                 {
                     m_frame->SelectUnit( symbol, unit );
-                    m_toolMgr->RunAction( ACTIONS::refreshPreview );
+                    m_toolMgr->PostAction( ACTIONS::refreshPreview );
                 }
             }
         }
@@ -891,7 +882,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
     } while( ( evt = Wait() ) ); //Should be assignment not equality test
 
     // Create a selection of original selection, drag selected/changed items, and new
-    // bend lines for later before we clear them in the commit. We'll need these
+    // bend lines for later before we clear them in the aCommit. We'll need these
     // to check for new junctions needed, etc.
     EE_SELECTION selectionCopy( selection );
 
@@ -902,28 +893,42 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
         selectionCopy.Add( line );
 
     // Save whatever new bend lines and changed lines survived the drag
-    commitDragLines();
+    for( SCH_LINE* newLine : m_newDragLines )
+    {
+        newLine->ClearEditFlags();
+        aCommit->Added( newLine, m_frame->GetScreen() );
+    }
+
+    // These lines have been changed, but aren't selected. We need
+    // to manually clear these edit flags or they'll stick around.
+    for( SCH_LINE* oldLine : m_changedDragLines )
+        oldLine->ClearEditFlags();
+
+    m_newDragLines.clear();
+    m_changedDragLines.clear();
 
     controls->ForceCursorPosition( false );
     controls->ShowCursor( false );
     controls->SetAutoPan( false );
 
-    if( !chain_commands )
-        m_moveOffset = { 0, 0 };
-
+    m_moveOffset = { 0, 0 };
     m_anchorPos.reset();
 
     if( restore_state )
     {
         m_selectionTool->RemoveItemsFromSel( &m_dragAdditions, QUIET_MODE );
-        m_frame->RollbackSchematicFromUndo();
     }
     else
     {
         // One last update after exiting loop (for slower stuff, such as updating SCREEN's RTree).
         for( EDA_ITEM* item : selection )
+        {
             updateItem( item, true );
 
+            if( SCH_ITEM* sch_item = dynamic_cast<SCH_ITEM*>( item ) )
+                sch_item->SetConnectivityDirty( true );
+        }
+
         if( selection.GetSize() == 1 && selection.Front()->IsNew() )
             m_frame->SaveCopyForRepeatItem( static_cast<SCH_ITEM*>( selection.Front() ) );
 
@@ -934,24 +939,23 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
         for( const DANGLING_END_ITEM& it : internalPoints )
         {
             if( m_frame->GetScreen()->IsExplicitJunctionNeeded( it.GetPosition()) )
-                m_frame->AddJunction( m_frame->GetScreen(), it.GetPosition(), true, false );
+                m_frame->AddJunction( aCommit, m_frame->GetScreen(), it.GetPosition() );
         }
 
         SCH_LINE_WIRE_BUS_TOOL* lwbTool = m_toolMgr->GetTool<SCH_LINE_WIRE_BUS_TOOL>();
-        lwbTool->TrimOverLappingWires( &selectionCopy );
-        lwbTool->AddJunctionsIfNeeded( &selectionCopy );
+        lwbTool->TrimOverLappingWires( aCommit, &selectionCopy );
+        lwbTool->AddJunctionsIfNeeded( aCommit, &selectionCopy );
 
         // This needs to run prior to `RecalculateConnections` because we need to identify
         // the lines that are newly dangling
-        if( m_isDrag && !isSlice )
-            trimDanglingLines();
+        if( m_isDrag && !aIsSlice )
+            trimDanglingLines( aCommit );
 
         // Auto-rotate any moved labels
         for( EDA_ITEM* item : selection )
             m_frame->AutoRotateItem( m_frame->GetScreen(), static_cast<SCH_ITEM*>( item ) );
 
-        m_frame->SchematicCleanUp();
-        m_frame->OnModify();
+        m_frame->SchematicCleanUp( aCommit );
     }
 
     for( EDA_ITEM* item : m_frame->GetScreen()->Items() )
@@ -963,7 +967,7 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
         item->ClearEditFlags();
 
     if( unselect )
-        m_toolMgr->RunAction( EE_ACTIONS::clearSelection, true );
+        m_toolMgr->RunAction( EE_ACTIONS::clearSelection );
     else
         m_selectionTool->RebuildSelection();  // Schematic cleanup might have merged lines, etc.
 
@@ -972,14 +976,14 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
     m_moveInProgress = false;
     m_frame->PopTool( aEvent );
 
-    return 0;
+    return !restore_state;
 }
 
 
-void SCH_MOVE_TOOL::trimDanglingLines()
+void SCH_MOVE_TOOL::trimDanglingLines( SCH_COMMIT* aCommit )
 {
     // Need a local cleanup first to ensure we remove unneeded junctions
-    m_frame->SchematicCleanUp( m_frame->GetScreen() );
+    m_frame->SchematicCleanUp( aCommit, m_frame->GetScreen() );
 
     std::set<SCH_ITEM*> danglers;
 
@@ -1003,10 +1007,9 @@ void SCH_MOVE_TOOL::trimDanglingLines()
     for( SCH_ITEM* line : danglers )
     {
         line->SetFlags( STRUCT_DELETED );
-        saveCopyInUndoList( line, UNDO_REDO::DELETED, true );
+        aCommit->Removed( line, m_frame->GetScreen() );
 
         updateItem( line, false );
-
         m_frame->RemoveFromScreen( line, m_frame->GetScreen() );
     }
 }
@@ -1141,8 +1144,8 @@ void SCH_MOVE_TOOL::getConnectedItems( SCH_ITEM* aOriginalItem, const VECTOR2I&
 }
 
 
-void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR2I& aPoint,
-                                           EDA_ITEMS& aList, bool& aAppendUndo )
+void SCH_MOVE_TOOL::getConnectedDragItems( SCH_COMMIT* aCommit, SCH_ITEM* aSelectedItem,
+                                           const VECTOR2I& aPoint, EDA_ITEMS& aList )
 {
     EE_RTREE&         items = m_frame->GetScreen()->Items();
     EE_RTREE::EE_TYPE itemsOverlappingRTree = items.Overlapping( aSelectedItem->GetBoundingBox() );
@@ -1150,7 +1153,8 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
     bool              ptHasUnselectedJunction = false;
 
     auto makeNewWire =
-            [&]( SCH_ITEM* fixed, SCH_ITEM* selected, const VECTOR2I& start, const VECTOR2I& end )
+            [this]( SCH_COMMIT* commit, SCH_ITEM* fixed, SCH_ITEM* selected, const VECTOR2I& start,
+                    const VECTOR2I& end )
             {
                 SCH_LINE* newWire;
 
@@ -1176,12 +1180,13 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
 
                 newWire->SetEndPoint( end );
                 m_frame->AddToScreen( newWire, m_frame->GetScreen() );
+                commit->Added( newWire, m_frame->GetScreen() );
 
                 return newWire;
             };
 
     auto makeNewJunction =
-            [&]( SCH_LINE* line, const VECTOR2I& pt )
+            [this]( SCH_COMMIT* commit, SCH_LINE* line, const VECTOR2I& pt )
             {
                 SCH_JUNCTION* junction = new SCH_JUNCTION( pt );
                 junction->SetFlags( IS_NEW );
@@ -1192,6 +1197,7 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
                     junction->SetLayer( LAYER_BUS_JUNCTION );
 
                 m_frame->AddToScreen( junction, m_frame->GetScreen() );
+                commit->Added( junction, m_frame->GetScreen() );
 
                 return junction;
             };
@@ -1283,34 +1289,22 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
                     if( line->HitTest( aPoint, 1 ) && !line->HasFlag( SELECTED )
                         && !line->HasFlag( SELECTED_BY_DRAG ) )
                     {
-                        newWire = makeNewWire( line, aSelectedItem, aPoint, aPoint );
+                        newWire = makeNewWire( aCommit, line, aSelectedItem, aPoint, aPoint );
                         newWire->SetFlags( SELECTED_BY_DRAG | STARTPOINT );
                         newWire->StoreAngle( ( line->Angle() + ANGLE_90 ).Normalize() );
                         aList.push_back( newWire );
 
-                        saveCopyInUndoList( newWire, UNDO_REDO::NEWITEM, aAppendUndo );
-                        aAppendUndo = true;
-
                         if( aPoint != line->GetStartPoint() && aPoint != line->GetEndPoint() )
                         {
                             // Split line in half
                             if( !line->IsNew() )
-                            {
-                                saveCopyInUndoList( line, UNDO_REDO::CHANGED, aAppendUndo );
-                                aAppendUndo = true;
-                            }
+                                aCommit->Modify( line, m_frame->GetScreen() );
 
                             VECTOR2I oldEnd = line->GetEndPoint();
                             line->SetEndPoint( aPoint );
 
-                            SCH_LINE*     secondHalf = makeNewWire( line, line, aPoint, oldEnd );
-                            SCH_JUNCTION* junction = makeNewJunction( line, aPoint );
-
-                            saveCopyInUndoList( secondHalf, UNDO_REDO::NEWITEM, aAppendUndo );
-                            aAppendUndo = true;
-
-                            saveCopyInUndoList( junction, UNDO_REDO::NEWITEM, aAppendUndo );
-                            aAppendUndo = true;
+                            makeNewWire( aCommit, line, line, aPoint, oldEnd );
+                            makeNewJunction( aCommit, line, aPoint );
                         }
                         else
                         {
@@ -1368,12 +1362,9 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
                     {
                         // Add a new wire between the sheetpin and the selected item so the
                         // selected item can be dragged.
-                        newWire = makeNewWire( pin, aSelectedItem, aPoint, aPoint );
+                        newWire = makeNewWire( aCommit, pin, aSelectedItem, aPoint, aPoint );
                         newWire->SetFlags( SELECTED_BY_DRAG | STARTPOINT );
                         aList.push_back( newWire );
-
-                        saveCopyInUndoList( newWire, UNDO_REDO::NEWITEM, aAppendUndo );
-                        aAppendUndo = true;
                     }
                 }
             }
@@ -1386,12 +1377,9 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
             {
                 // Add a new wire between the symbol or junction and the selected item so
                 // the selected item can be dragged.
-                newWire = makeNewWire( test, aSelectedItem, aPoint, aPoint );
+                newWire = makeNewWire( aCommit, test, aSelectedItem, aPoint, aPoint );
                 newWire->SetFlags( SELECTED_BY_DRAG | STARTPOINT );
                 aList.push_back( newWire );
-
-                saveCopyInUndoList( newWire, UNDO_REDO::NEWITEM, aAppendUndo );
-                aAppendUndo = true;
             }
 
             break;
@@ -1452,12 +1440,9 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
             {
                 // Add a new wire between the label and the selected item so the selected item
                 // can be dragged.
-                newWire = makeNewWire( test, aSelectedItem, aPoint, aPoint );
+                newWire = makeNewWire( aCommit, test, aSelectedItem, aPoint, aPoint );
                 newWire->SetFlags( SELECTED_BY_DRAG | STARTPOINT );
                 aList.push_back( newWire );
-
-                saveCopyInUndoList( newWire, UNDO_REDO::NEWITEM, aAppendUndo );
-                aAppendUndo = true;
             }
 
             break;
@@ -1497,7 +1482,7 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
                         else
                             otherEnd = ends[0];
 
-                        getConnectedDragItems( test, otherEnd, aList, aAppendUndo );
+                        getConnectedDragItems( aCommit, test, otherEnd, aList );
 
                         // No need to test the other end of the bus entry
                         break;
@@ -1559,7 +1544,7 @@ void SCH_MOVE_TOOL::moveItem( EDA_ITEM* aItem, const VECTOR2I& aDelta )
         SCH_SHEET_PIN* pin = (SCH_SHEET_PIN*) aItem;
 
         pin->SetStoredPos( pin->GetStoredPos() + aDelta );
-        pin->ConstrainOnEdge( pin->GetStoredPos() );
+        pin->ConstrainOnEdge( pin->GetStoredPos(), true );
         break;
     }
 
@@ -1589,22 +1574,22 @@ void SCH_MOVE_TOOL::moveItem( EDA_ITEM* aItem, const VECTOR2I& aDelta )
         break;
     }
 
-    getView()->Hide( aItem, false );
     aItem->SetFlags( IS_MOVING );
 }
 
 
 int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
 {
-    EE_GRID_HELPER grid( m_toolMgr);
-    EE_SELECTION& selection = m_selectionTool->RequestSelection( EE_COLLECTOR::MovableItems );
-    bool appendUndo = false;
+    EE_GRID_HELPER    grid( m_toolMgr);
+    EE_SELECTION&     selection = m_selectionTool->RequestSelection( EE_COLLECTOR::MovableItems );
+    GRID_HELPER_GRIDS selectionGrid = grid.GetSelectionGrid( selection );
+    SCH_COMMIT        commit( m_toolMgr );
 
     auto doMoveItem =
             [&]( EDA_ITEM* item, const VECTOR2I& delta )
             {
-                saveCopyInUndoList( item, UNDO_REDO::CHANGED, appendUndo );
-                appendUndo = true;
+                commit.Modify( item, m_frame->GetScreen() );
+
                 // Ensure only one end is moved when calling moveItem
                 // i.e. we are in drag mode
                 bool tmp_isDrag = m_isDrag;
@@ -1647,70 +1632,72 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
                 line->ClearFlags();
                 line->SetFlags( SELECTED );
                 line->SetFlags( flags[ii] );
-                getConnectedDragItems( line, pts[ii], drag_items, appendUndo );
+                getConnectedDragItems( &commit, line, pts[ii], drag_items );
                 std::set<EDA_ITEM*> unique_items( drag_items.begin(), drag_items.end() );
 
-                VECTOR2I gridpt = grid.AlignGrid( pts[ii] ) - pts[ii];
+                VECTOR2I delta = grid.AlignGrid( pts[ii], selectionGrid ) - pts[ii];
 
-                if( gridpt != VECTOR2I( 0, 0 ) )
+                if( delta != VECTOR2I( 0, 0 ) )
                 {
                     for( EDA_ITEM* dragItem : unique_items )
                     {
                         if( dragItem->GetParent() && dragItem->GetParent()->IsSelected() )
                             continue;
 
-                        doMoveItem( dragItem, gridpt );
+                        doMoveItem( dragItem, delta );
                     }
                 }
             }
         }
-        else if( item->Type() == SCH_FIELD_T )
+        else if( item->Type() == SCH_FIELD_T || item->Type() == SCH_TEXT_T )
         {
-            VECTOR2I gridpt = grid.AlignGrid( item->GetPosition() ) - item->GetPosition();
+            VECTOR2I delta = grid.AlignGrid( item->GetPosition(), selectionGrid ) - item->GetPosition();
 
-            if( gridpt != VECTOR2I( 0, 0 ) )
-                doMoveItem( item, gridpt );
+            if( delta != VECTOR2I( 0, 0 ) )
+                doMoveItem( item, delta );
         }
         else if( item->Type() == SCH_SHEET_T )
         {
             SCH_SHEET* sheet = static_cast<SCH_SHEET*>( item );
             VECTOR2I   topLeft = sheet->GetPosition();
             VECTOR2I   bottomRight = topLeft + sheet->GetSize();
-            VECTOR2I   tl_gridpt = grid.AlignGrid( topLeft ) - topLeft;
-            VECTOR2I   br_gridpt = grid.AlignGrid( bottomRight ) - bottomRight;
+            VECTOR2I   tl_delta = grid.AlignGrid( topLeft, selectionGrid ) - topLeft;
+            VECTOR2I   br_delta = grid.AlignGrid( bottomRight, selectionGrid ) - bottomRight;
 
-            if( tl_gridpt != VECTOR2I( 0, 0 ) || br_gridpt != VECTOR2I( 0, 0 ) )
+            if( tl_delta != VECTOR2I( 0, 0 ) || br_delta != VECTOR2I( 0, 0 ) )
             {
-                doMoveItem( sheet, tl_gridpt );
+                doMoveItem( sheet, tl_delta );
 
-                VECTOR2I newSize = (VECTOR2I) sheet->GetSize() - tl_gridpt + br_gridpt;
-                sheet->SetSize( wxSize( newSize.x, newSize.y ) );
+                VECTOR2I newSize = (VECTOR2I) sheet->GetSize() - tl_delta + br_delta;
+                sheet->SetSize( VECTOR2I( newSize.x, newSize.y ) );
                 updateItem( sheet, true );
+            }
 
-                for( SCH_SHEET_PIN* pin : sheet->GetPins() )
+            for( SCH_SHEET_PIN* pin : sheet->GetPins() )
+            {
+                VECTOR2I newPos;
+
+                if( pin->GetSide() == SHEET_SIDE::TOP || pin->GetSide() == SHEET_SIDE::LEFT )
+                    newPos = pin->GetPosition() + tl_delta;
+                else
+                    newPos = pin->GetPosition() + br_delta;
+
+                VECTOR2I delta = grid.AlignGrid( newPos, selectionGrid ) - pin->GetPosition();
+
+                if( delta != VECTOR2I( 0, 0 ) )
                 {
-                    VECTOR2I gridpt;
+                    EDA_ITEMS drag_items;
+                    getConnectedDragItems( &commit, pin, pin->GetConnectionPoints()[0],
+                                           drag_items );
 
-                    if( pin->GetSide() == SHEET_SIDE::TOP || pin->GetSide() == SHEET_SIDE::LEFT )
-                        gridpt = tl_gridpt;
-                    else
-                        gridpt = br_gridpt;
+                    doMoveItem( pin, delta );
 
-                    if( gridpt != VECTOR2I( 0, 0 ) )
+                    for( EDA_ITEM* dragItem : drag_items )
                     {
-                        EDA_ITEMS drag_items;
-                        getConnectedDragItems( pin, pin->GetConnectionPoints()[0], drag_items,
-                                               appendUndo );
+                        if( dragItem->GetParent() && dragItem->GetParent()->IsSelected() )
+                            continue;
 
-                        doMoveItem( pin, gridpt );
-
-                        for( EDA_ITEM* dragItem : drag_items )
-                        {
-                            if( dragItem->GetParent() && dragItem->GetParent()->IsSelected() )
-                                continue;
-
-                            doMoveItem( dragItem, gridpt );
-                        }
+                        doMoveItem( dragItem, delta );
                     }
                 }
             }
@@ -1719,10 +1706,10 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
         {
             SCH_ITEM*             schItem = static_cast<SCH_ITEM*>( item );
             std::vector<VECTOR2I> connections = schItem->GetConnectionPoints();
-            EDA_ITEMS             drag_items{ item };
+            EDA_ITEMS             drag_items;
 
             for( const VECTOR2I& point : connections )
-                getConnectedDragItems( schItem, point, drag_items, appendUndo );
+                getConnectedDragItems( &commit, schItem, point, drag_items );
 
             std::map<VECTOR2I, int> shifts;
             VECTOR2I                most_common( 0, 0 );
@@ -1730,7 +1717,7 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
 
             for( const VECTOR2I& conn : connections )
             {
-                VECTOR2I gridpt = grid.AlignGrid( conn ) - conn;
+                VECTOR2I gridpt = grid.AlignGrid( conn, selectionGrid ) - conn;
 
                 shifts[gridpt]++;
 
@@ -1743,6 +1730,8 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
 
             if( most_common != VECTOR2I( 0, 0 ) )
             {
+                doMoveItem( item, most_common );
+
                 for( EDA_ITEM* dragItem : drag_items )
                 {
                     if( dragItem->GetParent() && dragItem->GetParent()->IsSelected() )
@@ -1755,39 +1744,17 @@ int SCH_MOVE_TOOL::AlignElements( const TOOL_EVENT& aEvent )
     }
 
     SCH_LINE_WIRE_BUS_TOOL* lwbTool = m_toolMgr->GetTool<SCH_LINE_WIRE_BUS_TOOL>();
-    lwbTool->TrimOverLappingWires( &selection );
-    lwbTool->AddJunctionsIfNeeded( &selection );
+    lwbTool->TrimOverLappingWires( &commit, &selection );
+    lwbTool->AddJunctionsIfNeeded( &commit, &selection );
 
     m_toolMgr->PostEvent( EVENTS::SelectedItemsMoved );
 
-    m_frame->SchematicCleanUp();
-    m_frame->TestDanglingEnds();
-
-    m_frame->OnModify();
+    m_frame->SchematicCleanUp( &commit );
+    commit.Push( _( "Align" ) );
     return 0;
 }
 
 
-void SCH_MOVE_TOOL::commitDragLines()
-{
-    for( SCH_LINE* newLine : m_newDragLines )
-    {
-        newLine->ClearEditFlags();
-        saveCopyInUndoList( newLine, UNDO_REDO::NEWITEM, true );
-    }
-
-    // These lines have been changed, but aren't selected. We need
-    // to manually clear these edit flags or they'll stick around.
-    for( SCH_LINE* oldLine : m_changedDragLines )
-    {
-        oldLine->ClearEditFlags();
-    }
-
-    m_newDragLines.clear();
-    m_changedDragLines.clear();
-}
-
-
 void SCH_MOVE_TOOL::clearNewDragLines()
 {
     // Remove new bend lines added during the drag
@@ -1803,7 +1770,6 @@ void SCH_MOVE_TOOL::clearNewDragLines()
 
 void SCH_MOVE_TOOL::setTransitions()
 {
-    Go( &SCH_MOVE_TOOL::Main,               EE_ACTIONS::moveActivate.MakeEvent() );
     Go( &SCH_MOVE_TOOL::Main,               EE_ACTIONS::move.MakeEvent() );
     Go( &SCH_MOVE_TOOL::Main,               EE_ACTIONS::drag.MakeEvent() );
     Go( &SCH_MOVE_TOOL::AlignElements,      EE_ACTIONS::alignToGrid.MakeEvent() );
