@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2014-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2014-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,9 +36,55 @@
 #include <math/vector2d.h>  // for VECTOR2I
 #include <trigo.h>
 
-// Returns true if the point P is on the segment S.
-// faster than TestSegmentHit() because P should be exactly on S
-// therefore works fine only for H, V and 45 deg segm (suitable for wires in eeschema)
+
+/*
+CircleCenterFrom3Points calculate the center of a circle defined by 3 points
+It is similar to CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, const VECTOR2D& aEnd )
+but it was needed to debug CalcArcCenter, so I keep it available for other issues in CalcArcCenter
+
+The perpendicular bisector of the segment between two points is the
+set of all points equidistant from both.  So if you take the
+perpendicular bisector of (x1,y1) and (x2,y2) and the perpendicular
+bisector of the segment from (x2,y2) to (x3,y3) and find the
+intersection of those lines, that point will be the center.
+
+To find the equation of the perpendicular bisector of (x1,y1) to (x2,y2),
+you know that it passes through the midpoint of the segment:
+((x1+x2)/2,(y1+y2)/2), and if the slope of the line
+connecting (x1,y1) to (x2,y2) is m, the slope of the perpendicular
+bisector is -1/m.  Work out the equations for the two lines, find
+their intersection, and bingo!  You've got the coordinates of the center.
+
+An error should occur if the three points lie on a line, and you'll
+need special code to check for the case where one of the slopes is zero.
+
+see https://web.archive.org/web/20171223103555/http://mathforum.org/library/drmath/view/54323.html
+*/
+
+//#define USE_ALTERNATE_CENTER_ALGO
+
+#ifdef USE_ALTERNATE_CENTER_ALGO
+bool CircleCenterFrom3Points( const VECTOR2D& p1, const VECTOR2D& p2,  const VECTOR2D& p3, VECTOR2D* aCenter )
+{
+    // Move coordinate origin to p2, to simplify calculations
+    VECTOR2D b = p1 - p2;
+    VECTOR2D d = p3 - p2;
+    double bc = ( b.x*b.x + b.y*b.y ) / 2.0;
+    double cd = ( -d.x*d.x - d.y*d.y ) / 2.0;
+    double det = -b.x*d.y + d.x*b.y;
+
+    if( fabs(det) < 1.0e-6 )     // arbitrary limit to avoid divide by 0
+        return false;
+
+    det = 1/det;
+    aCenter->x = ( -bc*d.y - cd*b.y ) * det;
+    aCenter->y = ( b.x*cd + d.x*bc ) * det;
+    *aCenter += p2;
+
+    return true;
+}
+#endif
+
 bool IsPointOnSegment( const VECTOR2I& aSegStart, const VECTOR2I& aSegEnd,
                        const VECTOR2I& aTestPoint )
 {
@@ -47,7 +93,7 @@ bool IsPointOnSegment( const VECTOR2I& aSegStart, const VECTOR2I& aSegEnd,
 
     // Use long long here to avoid overflow in calculations
     if( (long long) vectSeg.x * vectPoint.y - (long long) vectSeg.y * vectPoint.x )
-        return false;        /* Cross product non-zero, vectors not parallel */
+        return false;         /* Cross product non-zero, vectors not parallel */
 
     if( ( (long long) vectSeg.x * vectPoint.x + (long long) vectSeg.y * vectPoint.y ) <
         ( (long long) vectPoint.x * vectPoint.x + (long long) vectPoint.y * vectPoint.y ) )
@@ -57,19 +103,18 @@ bool IsPointOnSegment( const VECTOR2I& aSegStart, const VECTOR2I& aSegEnd,
 }
 
 
-// Returns true if the segment 1 intersected the segment 2.
 bool SegmentIntersectsSegment( const VECTOR2I& a_p1_l1, const VECTOR2I& a_p2_l1,
                                const VECTOR2I& a_p1_l2, const VECTOR2I& a_p2_l2,
                                VECTOR2I* aIntersectionPoint )
 {
 
-    //We are forced to use 64bit ints because the internal units can overflow 32bit ints when
+    // We are forced to use 64bit ints because the internal units can overflow 32bit ints when
     // multiplied with each other, the alternative would be to scale the units down (i.e. divide
     // by a fixed number).
     int64_t dX_a, dY_a, dX_b, dY_b, dX_ab, dY_ab;
     int64_t num_a, num_b, den;
 
-    //Test for intersection within the bounds of both line segments using line equations of the
+    // Test for intersection within the bounds of both line segments using line equations of the
     // form:
     // x_k(u_k) = u_k * dX_k + x_k(0)
     // y_k(u_k) = u_k * dY_k + y_k(0)
@@ -84,14 +129,14 @@ bool SegmentIntersectsSegment( const VECTOR2I& a_p1_l1, const VECTOR2I& a_p2_l1,
 
     den   = dY_a  * dX_b - dY_b * dX_a ;
 
-    //Check if lines are parallel
+    // Check if lines are parallel.
     if( den == 0 )
         return false;
 
     num_a = dY_ab * dX_b - dY_b * dX_ab;
     num_b = dY_ab * dX_a - dY_a * dX_ab;
 
-    // Only compute the intersection point if requested
+    // Only compute the intersection point if requested.
     if( aIntersectionPoint )
     {
         *aIntersectionPoint = a_p1_l1;
@@ -106,19 +151,19 @@ bool SegmentIntersectsSegment( const VECTOR2I& a_p1_l1, const VECTOR2I& a_p2_l1,
         num_b = -num_b;
     }
 
-    //Test sign( u_a ) and return false if negative
+    // Test sign( u_a ) and return false if negative.
     if( num_a < 0 )
         return false;
 
-    //Test sign( u_b ) and return false if negative
+    // Test sign( u_b ) and return false if negative.
     if( num_b < 0 )
         return false;
 
-    //Test to ensure (u_a <= 1)
+    // Test to ensure (u_a <= 1).
     if( num_a > den )
         return false;
 
-    //Test to ensure (u_b <= 1)
+    // Test to ensure (u_b <= 1).
     if( num_b > den )
         return false;
 
@@ -141,14 +186,14 @@ bool TestSegmentHit( const VECTOR2I& aRefPoint, const VECTOR2I& aStart, const VE
     if( ymax < ymin )
         std::swap( ymax, ymin );
 
-    // First, check if we are outside of the bounding box
+    // Check if we are outside of the bounding box.
     if( ( ymin - aRefPoint.y > aDist ) || ( aRefPoint.y - ymax > aDist ) )
         return false;
 
     if( ( xmin - aRefPoint.x > aDist ) || ( aRefPoint.x - xmax > aDist ) )
         return false;
 
-    // Next, eliminate easy cases
+    // Eliminate easy cases.
     if( aStart.x == aEnd.x && aRefPoint.y > ymin && aRefPoint.y < ymax )
         return std::abs( delta.x ) <= aDist;
 
@@ -232,20 +277,6 @@ void RotatePoint( int* pX, int* pY, int cx, int cy, const EDA_ANGLE& angle )
 }
 
 
-void RotatePoint( wxPoint* point, const wxPoint& centre, const EDA_ANGLE& angle )
-{
-    int ox, oy;
-
-    ox = point->x - centre.x;
-    oy = point->y - centre.y;
-
-    RotatePoint( &ox, &oy, angle );
-
-    point->x = ox + centre.x;
-    point->y = oy + centre.y;
-}
-
-
 void RotatePoint( double* pX, double* pY, double cx, double cy, const EDA_ANGLE& angle )
 {
     double ox, oy;
@@ -302,8 +333,8 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aEnd,
                               const EDA_ANGLE& aAngle )
 {
     EDA_ANGLE angle( aAngle );
-    VECTOR2I  start = aStart;
-    VECTOR2I  end = aEnd;
+    VECTOR2D  start = aStart;
+    VECTOR2D  end = aEnd;
 
     if( angle < ANGLE_0 )
     {
@@ -325,8 +356,8 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aEnd,
     if( d_squared > 0.0 )
         d = sqrt( d_squared );
 
-    VECTOR2D vec2 = (end - start).Resize( d );
-    VECTOR2D vc = (end - start).Resize( chord / 2 );
+    VECTOR2D vec2 = VECTOR2D(end - start).Resize( d );
+    VECTOR2D vc = VECTOR2D(end - start).Resize( chord / 2 );
 
     RotatePoint( vec2, -ANGLE_90 );
 
@@ -337,6 +368,7 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aEnd,
 const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, const VECTOR2D& aEnd )
 {
     VECTOR2D center;
+
     double yDelta_21 = aMid.y - aStart.y;
     double xDelta_21 = aMid.x - aStart.x;
     double yDelta_32 = aEnd.y - aMid.y;
@@ -370,8 +402,8 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, cons
     {
         if( aStart == aEnd )
         {
-            // This is a special case for a 360 degrees arc.  In this case, the center is halfway between
-            // the midpoint and either end point
+            // This is a special case for a 360 degrees arc.  In this case, the center is
+            // halfway between the midpoint and either end point.
             center.x = ( aStart.x + aMid.x ) / 2.0;
             center.y = ( aStart.y + aMid.y ) / 2.0 ;
             return center;
@@ -385,10 +417,19 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, cons
             bSlope -= std::numeric_limits<double>::epsilon();
         }
     }
+#ifdef USE_ALTERNATE_CENTER_ALGO
+    // We can call ArcCenterFrom3Points from here because special cases are filtered.
+    CircleCenterFrom3Points( aStart, aMid, aEnd, &center );
+    return center;
+#endif
 
     // Prevent divide by zero error
+    // a small value is used. std::numeric_limits<double>::epsilon() is too small and
+    // generate false results
     if( aSlope == 0.0 )
-        aSlope = std::numeric_limits<double>::epsilon();
+        aSlope = 1e-10;
+    if( bSlope == 0.0 )
+        bSlope = 1e-10;
 
     // What follows is the calculation of the center using the slope of the two lines as well as
     // the propagated error that occurs when rounding to the nearest nanometer.  The error can be
@@ -397,22 +438,24 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, cons
     // to the standard deviation.
     // We ignore the possible covariance between variables.  We also truncate our series expansion
     // at the first term.  These are reasonable assumptions as the worst-case scenario is that we
-    // underestimate the potential uncertainty, which would potentially put us back at the status quo
+    // underestimate the potential uncertainty, which would potentially put us back at the status
+    // quo.
     double abSlopeStartEndY = aSlope * bSlope * ( aStart.y - aEnd.y );
-    double dabSlopeStartEndY = abSlopeStartEndY * std::sqrt( ( daSlope / aSlope * daSlope / aSlope )
-                                                           + ( dbSlope / bSlope * dbSlope / bSlope )
-                                                           + ( M_SQRT1_2 / ( aStart.y - aEnd.y )
-                                                               * M_SQRT1_2 / ( aStart.y - aEnd.y ) ) );
+    double dabSlopeStartEndY = abSlopeStartEndY *
+                               std::sqrt( ( daSlope / aSlope * daSlope / aSlope )
+                                        + ( dbSlope / bSlope * dbSlope / bSlope )
+                                        + ( M_SQRT1_2 / ( aStart.y - aEnd.y )
+                                          * M_SQRT1_2 / ( aStart.y - aEnd.y ) ) );
 
     double bSlopeStartMidX = bSlope * ( aStart.x + aMid.x );
     double dbSlopeStartMidX = bSlopeStartMidX * std::sqrt( ( dbSlope / bSlope * dbSlope / bSlope )
                                                          + ( M_SQRT1_2 / ( aStart.x + aMid.x )
-                                                                 * M_SQRT1_2 / ( aStart.x + aMid.x ) ) );
+                                                           * M_SQRT1_2 / ( aStart.x + aMid.x ) ) );
 
     double aSlopeMidEndX = aSlope * ( aMid.x + aEnd.x );
     double daSlopeMidEndX = aSlopeMidEndX * std::sqrt( ( daSlope / aSlope * daSlope / aSlope )
                                                      + ( M_SQRT1_2 / ( aMid.x + aEnd.x )
-                                                             * M_SQRT1_2 / ( aMid.x + aEnd.x ) ) );
+                                                       * M_SQRT1_2 / ( aMid.x + aEnd.x ) ) );
 
     double twiceBASlopeDiff = 2 * ( bSlope - aSlope );
     double dtwiceBASlopeDiff = 2 * std::sqrt( dbSlope * dbSlope + daSlope * daSlope );
@@ -423,9 +466,10 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, cons
                                        + daSlopeMidEndX * daSlopeMidEndX );
 
     double centerX = ( abSlopeStartEndY + bSlopeStartMidX - aSlopeMidEndX ) / twiceBASlopeDiff;
-
-    double dCenterX = centerX * std::sqrt( ( dCenterNumeratorX / centerNumeratorX * dCenterNumeratorX / centerNumeratorX )
-                                         + ( dtwiceBASlopeDiff / twiceBASlopeDiff * dtwiceBASlopeDiff / twiceBASlopeDiff ) );
+    double dCenterX = centerX * std::sqrt( ( dCenterNumeratorX / centerNumeratorX *
+                                             dCenterNumeratorX / centerNumeratorX )
+                                         + ( dtwiceBASlopeDiff / twiceBASlopeDiff *
+                                             dtwiceBASlopeDiff / twiceBASlopeDiff ) );
 
 
     double centerNumeratorY = ( ( aStart.x + aMid.x ) / 2.0 - centerX );
@@ -433,7 +477,8 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, cons
 
     double centerFirstTerm = centerNumeratorY / aSlope;
     double dcenterFirstTermY = centerFirstTerm * std::sqrt(
-                                          ( dCenterNumeratorY/ centerNumeratorY * dCenterNumeratorY / centerNumeratorY )
+                                          ( dCenterNumeratorY/ centerNumeratorY *
+                                            dCenterNumeratorY / centerNumeratorY )
                                         + ( daSlope / aSlope * daSlope / aSlope ) );
 
     double centerY = centerFirstTerm + ( aStart.y + aMid.y ) / 2.0;
@@ -444,16 +489,19 @@ const VECTOR2D CalcArcCenter( const VECTOR2D& aStart, const VECTOR2D& aMid, cons
     double rounded10CenterX = std::floor( ( centerX + 5.0 ) / 10.0 ) * 10.0;
     double rounded10CenterY = std::floor( ( centerY + 5.0 ) / 10.0 ) * 10.0;
 
-    // The last step is to find the nice, round numbers near our baseline estimate and see if they are within our uncertainty
-    // range.  If they are, then we use this round value as the true value.  This is justified because ALL values within the
-    // uncertainty range are equally true.  Using a round number will make sure that we are on a multiple of 1mil or 100nm
+    // The last step is to find the nice, round numbers near our baseline estimate and see if
+    // they are within our uncertainty range.  If they are, then we use this round value as the
+    // true value.  This is justified because ALL values within the uncertainty range are equally
+    // true.  Using a round number will make sure that we are on a multiple of 1mil or 100nm
     // when calculating centers.
-    if( std::abs( rounded100CenterX - centerX ) < dCenterX && std::abs( rounded100CenterY - centerY ) < dCenterY )
+    if( std::abs( rounded100CenterX - centerX ) < dCenterX &&
+        std::abs( rounded100CenterY - centerY ) < dCenterY )
     {
         center.x = rounded100CenterX;
         center.y = rounded100CenterY;
     }
-    else if( std::abs( rounded10CenterX - centerX ) < dCenterX && std::abs( rounded10CenterY - centerY ) < dCenterY )
+    else if( std::abs( rounded10CenterX - centerX ) < dCenterX &&
+             std::abs( rounded10CenterY - centerY ) < dCenterY )
     {
         center.x = rounded10CenterX;
         center.y = rounded10CenterY;
@@ -489,4 +537,3 @@ const VECTOR2I CalcArcCenter( const VECTOR2I& aStart, const VECTOR2I& aMid, cons
     return iCenter;
 }
 
-
