@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2022 Mark Roszko <mark.roszko@gmail.com>
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -18,9 +18,17 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <wx/dir.h>
 #include "pcbnew_jobs_handler.h"
+#include <board_commit.h>
+#include <board_design_settings.h>
+#include <drc/drc_item.h>
+#include <drc/drc_report.h>
+#include <drawing_sheet/ds_data_model.h>
+#include <drawing_sheet/ds_proxy_view_item.h>
 #include <jobs/job_fp_export_svg.h>
 #include <jobs/job_fp_upgrade.h>
+#include <jobs/job_export_pcb_ipc2581.h>
 #include <jobs/job_export_pcb_gerber.h>
 #include <jobs/job_export_pcb_gerbers.h>
 #include <jobs/job_export_pcb_drill.h>
@@ -28,35 +36,47 @@
 #include <jobs/job_export_pcb_pdf.h>
 #include <jobs/job_export_pcb_pos.h>
 #include <jobs/job_export_pcb_svg.h>
-#include <jobs/job_export_pcb_step.h>
+#include <jobs/job_export_pcb_3d.h>
+#include <jobs/job_pcb_drc.h>
 #include <cli/exit_codes.h>
+#include <exporters/place_file_exporter.h>
+#include <exporters/step/exporter_step.h>
 #include <plotters/plotter_dxf.h>
 #include <plotters/plotter_gerber.h>
 #include <plotters/plotters_pslike.h>
-#include <exporters/place_file_exporter.h>
-#include <exporters/step/exporter_step.h>
+#include <tool/tool_manager.h>
+#include <tools/drc_tool.h>
+#include <gerber_jobfile_writer.h>
 #include "gerber_placefile_writer.h"
-#include <pgm_base.h>
-#include <pcbplot.h>
-#include <board_design_settings.h>
-#include <pad.h>
-#include <pcbnew_settings.h>
-#include <wx/crt.h>
-#include <wx/dir.h>
-#include <pcb_plot_svg.h>
 #include <gendrill_Excellon_writer.h>
 #include <gendrill_gerber_writer.h>
+#include <kiface_base.h>
+#include <macros.h>
+#include <pad.h>
+#include <pcb_marker.h>
+#include <project/project_file.h>
+#include <exporters/export_svg.h>
+#include <kiface_ids.h>
+#include <netlist_reader/pcb_netlist.h>
+#include <netlist_reader/netlist_reader.h>
+#include <pcbnew_settings.h>
+#include <pcbplot.h>
+#include <pgm_base.h>
+#include <pcb_io/kicad_sexpr/pcb_io_kicad_sexpr.h>
+#include <reporter.h>
+#include <string_utf8_map.h>
 #include <wildcards_and_files_ext.h>
-#include <plugins/kicad/pcb_plugin.h>
-#include <gerber_jobfile_writer.h>
+#include <export_vrml.h>
+#include <wx/wfstream.h>
+#include <wx/zipstrm.h>
 
 #include "pcbnew_scripting_helpers.h"
 
 
-PCBNEW_JOBS_HANDLER::PCBNEW_JOBS_HANDLER()
+PCBNEW_JOBS_HANDLER::PCBNEW_JOBS_HANDLER( KIWAY* aKiway ) :
+        JOB_DISPATCHER( aKiway )
 {
-    Register( "step",
-              std::bind( &PCBNEW_JOBS_HANDLER::JobExportStep, this, std::placeholders::_1 ) );
+    Register( "3d", std::bind( &PCBNEW_JOBS_HANDLER::JobExportStep, this, std::placeholders::_1 ) );
     Register( "svg", std::bind( &PCBNEW_JOBS_HANDLER::JobExportSvg, this, std::placeholders::_1 ) );
     Register( "dxf", std::bind( &PCBNEW_JOBS_HANDLER::JobExportDxf, this, std::placeholders::_1 ) );
     Register( "pdf", std::bind( &PCBNEW_JOBS_HANDLER::JobExportPdf, this, std::placeholders::_1 ) );
@@ -71,46 +91,119 @@ PCBNEW_JOBS_HANDLER::PCBNEW_JOBS_HANDLER()
               std::bind( &PCBNEW_JOBS_HANDLER::JobExportFpUpgrade, this, std::placeholders::_1 ) );
     Register( "fpsvg",
               std::bind( &PCBNEW_JOBS_HANDLER::JobExportFpSvg, this, std::placeholders::_1 ) );
+    Register( "drc", std::bind( &PCBNEW_JOBS_HANDLER::JobExportDrc, this, std::placeholders::_1 ) );
+    Register( "ipc2581",
+              std::bind( &PCBNEW_JOBS_HANDLER::JobExportIpc2581, this, std::placeholders::_1 ) );
 }
 
 
 int PCBNEW_JOBS_HANDLER::JobExportStep( JOB* aJob )
 {
-    JOB_EXPORT_PCB_STEP* aStepJob = dynamic_cast<JOB_EXPORT_PCB_STEP*>( aJob );
+    JOB_EXPORT_PCB_3D* aStepJob = dynamic_cast<JOB_EXPORT_PCB_3D*>( aJob );
 
     if( aStepJob == nullptr )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD* brd = LoadBoard( aStepJob->m_filename );
+    BOARD* brd = LoadBoard( aStepJob->m_filename, true );
+    brd->GetProject()->ApplyTextVars( aJob->GetVarOverrides() );
 
     if( aStepJob->m_outputFile.IsEmpty() )
     {
         wxFileName fn = brd->GetFileName();
         fn.SetName( fn.GetName() );
-        fn.SetExt( wxS( "step" ) );
+
+        switch( aStepJob->m_format )
+        {
+        case JOB_EXPORT_PCB_3D::FORMAT::VRML: fn.SetExt( FILEEXT::VrmlFileExtension );
+            break;
+        case JOB_EXPORT_PCB_3D::FORMAT::STEP: fn.SetExt( FILEEXT::StepFileExtension );
+            break;
+        case JOB_EXPORT_PCB_3D::FORMAT::GLB: fn.SetExt( FILEEXT::GltfBinaryFileExtension );
+            break;
+        default: return CLI::EXIT_CODES::ERR_UNKNOWN; // shouldnt have gotten here
+        }
 
         aStepJob->m_outputFile = fn.GetFullName();
     }
 
-    EXPORTER_STEP_PARAMS params;
-    params.m_includeExcludedBom = aStepJob->m_includeExcludedBom;
-    params.m_boardOutlinesChainingEpsilon = aStepJob->m_boardOutlinesChainingEpsilon;
-    params.m_overwrite = aStepJob->m_overwrite;
-    params.m_substModels = aStepJob->m_substModels;
-    params.m_origin = VECTOR2D( aStepJob->m_xOrigin, aStepJob->m_yOrigin );
-    params.m_useDrillOrigin = aStepJob->m_useDrillOrigin;
-    params.m_useGridOrigin = aStepJob->m_useGridOrigin;
-    params.m_boardOnly = aStepJob->m_boardOnly;
-
-    EXPORTER_STEP stepExporter( brd, params );
-    stepExporter.m_outputFile = aStepJob->m_outputFile;
-
-    if( !stepExporter.Export() )
+    if( aStepJob->m_format == JOB_EXPORT_PCB_3D::FORMAT::VRML )
     {
-        return CLI::EXIT_CODES::ERR_UNKNOWN;
+
+        double scale = 0.0;
+        switch ( aStepJob->m_vrmlUnits )
+        {
+        case JOB_EXPORT_PCB_3D::VRML_UNITS::MILLIMETERS: scale = 1.0; break;
+        case JOB_EXPORT_PCB_3D::VRML_UNITS::METERS: scale = 0.001; break;
+        case JOB_EXPORT_PCB_3D::VRML_UNITS::TENTHS: scale = 10.0 / 25.4; break;
+        case JOB_EXPORT_PCB_3D::VRML_UNITS::INCHES: scale = 1.0 / 25.4; break;
+        }
+
+        EXPORTER_VRML vrmlExporter( brd );
+        wxString      messages;
+
+        double originX = pcbIUScale.IUTomm( aStepJob->m_xOrigin );
+        double originY = pcbIUScale.IUTomm( aStepJob->m_yOrigin );
+
+        if( !aStepJob->m_hasUserOrigin )
+        {
+            BOX2I bbox = brd->ComputeBoundingBox( true );
+            originX = pcbIUScale.IUTomm( bbox.GetCenter().x );
+            originY = pcbIUScale.IUTomm( bbox.GetCenter().y );
+        }
+
+        bool success = vrmlExporter.ExportVRML_File(
+                brd->GetProject(), &messages, aStepJob->m_outputFile, scale,
+                !aStepJob->m_vrmlModelDir.IsEmpty(), aStepJob->m_vrmlRelativePaths,
+                aStepJob->m_vrmlModelDir, originX, originY );
+
+        if ( success )
+        {
+            m_reporter->Report( wxString::Format( _( "Successfully exported VRML to %s" ), aStepJob->m_outputFile ),
+                                RPT_SEVERITY_INFO );
+        }
+        else
+        {
+            m_reporter->Report( _( "Error exporting VRML" ), RPT_SEVERITY_ERROR );
+            return CLI::EXIT_CODES::ERR_UNKNOWN;
+        }
+    }
+    else
+    {
+        EXPORTER_STEP_PARAMS params;
+        params.m_exportTracks = aStepJob->m_exportTracks;
+        params.m_exportZones = aStepJob->m_exportZones;
+        params.m_includeUnspecified = aStepJob->m_includeUnspecified;
+        params.m_includeDNP = aStepJob->m_includeDNP;
+        params.m_BoardOutlinesChainingEpsilon = aStepJob->m_BoardOutlinesChainingEpsilon;
+        params.m_overwrite = aStepJob->m_overwrite;
+        params.m_substModels = aStepJob->m_substModels;
+        params.m_origin = VECTOR2D( aStepJob->m_xOrigin, aStepJob->m_yOrigin );
+        params.m_useDrillOrigin = aStepJob->m_useDrillOrigin;
+        params.m_useGridOrigin = aStepJob->m_useGridOrigin;
+        params.m_boardOnly = aStepJob->m_boardOnly;
+        params.m_optimizeStep = aStepJob->m_optimizeStep;
+
+        switch( aStepJob->m_format )
+        {
+        case JOB_EXPORT_PCB_3D::FORMAT::STEP:
+            params.m_format = EXPORTER_STEP_PARAMS::FORMAT::STEP;
+            break;
+        case JOB_EXPORT_PCB_3D::FORMAT::GLB:
+            params.m_format = EXPORTER_STEP_PARAMS::FORMAT::GLB;
+            break;
+        default: return CLI::EXIT_CODES::ERR_UNKNOWN; // shouldnt have gotten here
+        }
+
+        EXPORTER_STEP stepExporter( brd, params );
+        stepExporter.m_outputFile = aStepJob->m_outputFile;
+
+        if( !stepExporter.Export() )
+        {
+            return CLI::EXIT_CODES::ERR_UNKNOWN;
+        }
     }
 
     return CLI::EXIT_CODES::OK;
@@ -136,16 +229,18 @@ int PCBNEW_JOBS_HANDLER::JobExportSvg( JOB* aJob )
     svgPlotOptions.m_drillShapeOption = aSvgJob->m_drillShapeOption;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD* brd = LoadBoard( aSvgJob->m_filename );
+    BOARD* brd = LoadBoard( aSvgJob->m_filename, true );
+    loadOverrideDrawingSheet( brd, aSvgJob->m_drawingSheet );
+    brd->GetProject()->ApplyTextVars( aJob->GetVarOverrides() );
 
     if( aJob->IsCli() )
     {
-        if( PCB_PLOT_SVG::Plot( brd, svgPlotOptions ) )
-            wxPrintf( _( "Successfully created svg file" ) );
+        if( EXPORT_SVG::Plot( brd, svgPlotOptions ) )
+            m_reporter->Report( _( "Successfully created svg file" ) + wxS( "\n" ), RPT_SEVERITY_INFO );
         else
-            wxPrintf( _( "Error creating svg file" ) );
+            m_reporter->Report( _( "Error creating svg file" ) + wxS( "\n" ), RPT_SEVERITY_ERROR );
     }
 
     return CLI::EXIT_CODES::OK;
@@ -160,9 +255,11 @@ int PCBNEW_JOBS_HANDLER::JobExportDxf( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD* brd = LoadBoard( aDxfJob->m_filename );
+    BOARD* brd = LoadBoard( aDxfJob->m_filename, true );
+    loadOverrideDrawingSheet( brd, aDxfJob->m_drawingSheet );
+    brd->GetProject()->ApplyTextVars( aJob->GetVarOverrides() );
 
     if( aDxfJob->m_outputFile.IsEmpty() )
     {
@@ -197,7 +294,7 @@ int PCBNEW_JOBS_HANDLER::JobExportDxf( JOB* aJob )
 
     if( plotter )
     {
-        PlotBoardLayers( brd, plotter, aDxfJob->m_printMaskLayer.SeqStackupBottom2Top(), plotOpts );
+        PlotBoardLayers( brd, plotter, aDxfJob->m_printMaskLayer, plotOpts );
         plotter->EndPlot();
     }
 
@@ -215,9 +312,11 @@ int PCBNEW_JOBS_HANDLER::JobExportPdf( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD* brd = LoadBoard( aPdfJob->m_filename );
+    BOARD* brd = LoadBoard( aPdfJob->m_filename, true );
+    loadOverrideDrawingSheet( brd, aPdfJob->m_drawingSheet );
+    brd->GetProject()->ApplyTextVars( aJob->GetVarOverrides() );
 
     if( aPdfJob->m_outputFile.IsEmpty() )
     {
@@ -264,8 +363,8 @@ int PCBNEW_JOBS_HANDLER::JobExportPdf( JOB* aJob )
 
     if( plotter )
     {
-        PlotBoardLayers( brd, plotter, aPdfJob->m_printMaskLayer.SeqStackupBottom2Top(), plotOpts );
-        PlotInteractiveLayer( brd, plotter );
+        PlotBoardLayers( brd, plotter, aPdfJob->m_printMaskLayer, plotOpts );
+        PlotInteractiveLayer( brd, plotter, plotOpts );
         plotter->EndPlot();
     }
 
@@ -284,10 +383,12 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD*          brd = LoadBoard( aGerberJob->m_filename );
-    PCB_PLOT_PARAMS boardPlotOptions = brd->GetPlotOptions();
+    BOARD* brd = LoadBoard( aGerberJob->m_filename, true );
+    loadOverrideDrawingSheet( brd, aGerberJob->m_drawingSheet );
+
+    PCB_PLOT_PARAMS       boardPlotOptions = brd->GetPlotOptions();
     LSET                  plotOnAllLayersSelection = boardPlotOptions.GetPlotOnAllLayersSelection();
     GERBER_JOBFILE_WRITER jobfile_writer( brd );
 
@@ -295,20 +396,26 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
 
     if( aGerberJob->m_useBoardPlotParams )
     {
-        aGerberJob->m_printMaskLayer = boardPlotOptions.GetLayerSelection();
+        // The board plot options are saved with all copper layers enabled, even those that don't
+        // exist in the current stackup. This is done so the layers are automatically enabled in the plot
+        // dialog when the user enables them. We need to filter out these not-enabled layers here so
+        // we don't plot 32 layers when we only have 4, etc.
+        LSET plotLayers = ( boardPlotOptions.GetLayerSelection() & LSET::AllNonCuMask() )
+                          | ( brd->GetEnabledLayers() & LSET::AllCuMask() );
+        aGerberJob->m_printMaskLayer = plotLayers.SeqStackupForPlotting();
         aGerberJob->m_layersIncludeOnAll = boardPlotOptions.GetPlotOnAllLayersSelection();
     }
     else
     {
         // default to the board enabled layers
         if( aGerberJob->m_printMaskLayer == 0 )
-            aGerberJob->m_printMaskLayer = brd->GetEnabledLayers();
+            aGerberJob->m_printMaskLayer = brd->GetEnabledLayers().SeqStackupForPlotting();
 
         if( aGerberJob->m_layersIncludeOnAllSet )
             aGerberJob->m_layersIncludeOnAll = plotOnAllLayersSelection;
     }
 
-    for( LSEQ seq = aGerberJob->m_printMaskLayer.UIOrder(); seq; ++seq )
+    for( LSEQ seq = LSET( aGerberJob->m_printMaskLayer ).UIOrder(); seq; ++seq )
     {
         LSEQ plotSequence;
 
@@ -338,7 +445,7 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
         if( plotOpts.GetUseGerberProtelExtensions() )
             fileExt = GetGerberProtelExtension( layer );
         else
-            fileExt = GerberFileExtension;
+            fileExt = FILEEXT::GerberFileExtension;
 
         BuildPlotFileName( &fn, aGerberJob->m_outputFile, brd->GetLayerName( layer ), fileExt );
         wxString fullname = fn.GetFullName();
@@ -351,13 +458,16 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
 
         if( plotter )
         {
-            wxPrintf( _( "Plotted to '%s'.\n" ), fn.GetFullPath() );
+            m_reporter->Report( wxString::Format( _( "Plotted to '%s'.\n" ), fn.GetFullPath() ),
+                                RPT_SEVERITY_ACTION );
             PlotBoardLayers( brd, plotter, plotSequence, plotOpts );
             plotter->EndPlot();
         }
         else
         {
-            wxFprintf( stderr, _( "Failed to plot to '%s'.\n" ), fn.GetFullPath() );
+            m_reporter->Report( wxString::Format( _( "Failed to plot to '%s'.\n" ),
+                                                  fn.GetFullPath() ),
+                                RPT_SEVERITY_ERROR );
             exitCode = CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
         }
 
@@ -367,7 +477,8 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
     wxFileName fn( aGerberJob->m_filename );
 
     // Build gerber job file from basename
-    BuildPlotFileName( &fn, aGerberJob->m_outputFile, wxT( "job" ), GerberJobFileExtension );
+    BuildPlotFileName( &fn, aGerberJob->m_outputFile, wxT( "job" ),
+                       FILEEXT::GerberJobFileExtension );
     jobfile_writer.CreateJobFile( fn.GetFullPath() );
 
     return exitCode;
@@ -406,9 +517,10 @@ int PCBNEW_JOBS_HANDLER::JobExportGerber( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD* brd = LoadBoard( aGerberJob->m_filename );
+    BOARD* brd = LoadBoard( aGerberJob->m_filename, true );
+    brd->GetProject()->ApplyTextVars( aJob->GetVarOverrides() );
 
     if( aGerberJob->m_outputFile.IsEmpty() )
     {
@@ -425,18 +537,19 @@ int PCBNEW_JOBS_HANDLER::JobExportGerber( JOB* aJob )
 
     // We are feeding it one layer at the start here to silence a logic check
     GERBER_PLOTTER* plotter = (GERBER_PLOTTER*) StartPlotBoard(
-            brd, &plotOpts, aGerberJob->m_printMaskLayer.Seq().front(), aGerberJob->m_outputFile,
+            brd, &plotOpts, aGerberJob->m_printMaskLayer.front(), aGerberJob->m_outputFile,
             wxEmptyString, wxEmptyString );
 
     if( plotter )
     {
-        PlotBoardLayers( brd, plotter, aGerberJob->m_printMaskLayer.SeqStackupBottom2Top(),
-                         plotOpts );
+        PlotBoardLayers( brd, plotter, aGerberJob->m_printMaskLayer, plotOpts );
         plotter->EndPlot();
     }
     else
     {
-        wxFprintf( stderr, _( "Failed to plot to '%s'.\n" ), aGerberJob->m_outputFile );
+        m_reporter->Report( wxString::Format( _( "Failed to plot to '%s'.\n" ),
+                                              aGerberJob->m_outputFile ),
+                            RPT_SEVERITY_ERROR );
         exitCode = CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
     }
 
@@ -457,16 +570,15 @@ int PCBNEW_JOBS_HANDLER::JobExportDrill( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD* brd = LoadBoard( aDrillJob->m_filename );
+    BOARD* brd = LoadBoard( aDrillJob->m_filename, true );
 
     // ensure output dir exists
     wxFileName fn( aDrillJob->m_outputDir + wxT( "/" ) );
     if( !fn.Mkdir( wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL ) )
     {
-        if( aJob->IsCli() )
-            wxFprintf( stderr, _( "Failed to create output directory\n" ) );
+        m_reporter->Report( _( "Failed to create output directory\n" ), RPT_SEVERITY_ERROR );
         return CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
     }
 
@@ -483,7 +595,7 @@ int PCBNEW_JOBS_HANDLER::JobExportDrill( JOB* aJob )
 
     VECTOR2I offset;
 
-    if( aDrillJob->m_drillOrigin == JOB_EXPORT_PCB_DRILL::DRILL_ORIGIN::ABSOLUTE )
+    if( aDrillJob->m_drillOrigin == JOB_EXPORT_PCB_DRILL::DRILL_ORIGIN::ABS )
         offset = VECTOR2I( 0, 0 );
     else
         offset = brd->GetDesignSettings().GetAuxOrigin();
@@ -542,7 +654,7 @@ int PCBNEW_JOBS_HANDLER::JobExportDrill( JOB* aJob )
         excellonWriter->SetMapFileFormat( mapFormat );
 
         if( !excellonWriter->CreateDrillandMapFilesSet( aDrillJob->m_outputDir, true,
-                                                        aDrillJob->m_generateMap, this ) )
+                                                        aDrillJob->m_generateMap, m_reporter ) )
         {
             return CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
         }
@@ -562,7 +674,7 @@ int PCBNEW_JOBS_HANDLER::JobExportDrill( JOB* aJob )
         gerberWriter->SetMapFileFormat( mapFormat );
 
         if( !gerberWriter->CreateDrillandMapFilesSet( aDrillJob->m_outputDir, true,
-                                                      aDrillJob->m_generateMap, this ) )
+                                                      aDrillJob->m_generateMap, m_reporter ) )
         {
             return CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
         }
@@ -580,9 +692,9 @@ int PCBNEW_JOBS_HANDLER::JobExportPos( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading board\n" ) );
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
 
-    BOARD* brd = LoadBoard( aPosJob->m_filename );
+    BOARD* brd = LoadBoard( aPosJob->m_filename, true );
 
     if( aPosJob->m_outputFile.IsEmpty() )
     {
@@ -590,11 +702,11 @@ int PCBNEW_JOBS_HANDLER::JobExportPos( JOB* aJob )
         fn.SetName( fn.GetName() );
 
         if( aPosJob->m_format == JOB_EXPORT_PCB_POS::FORMAT::ASCII )
-            fn.SetExt( FootprintPlaceFileExtension );
+            fn.SetExt( FILEEXT::FootprintPlaceFileExtension );
         else if( aPosJob->m_format == JOB_EXPORT_PCB_POS::FORMAT::CSV )
-            fn.SetExt( CsvFileExtension );
+            fn.SetExt( FILEEXT::CsvFileExtension );
         else if( aPosJob->m_format == JOB_EXPORT_PCB_POS::FORMAT::GERBER )
-            fn.SetExt( GerberFileExtension );
+            fn.SetExt( FILEEXT::GerberFileExtension );
 
         aPosJob->m_outputFile = fn.GetFullName();
     }
@@ -619,6 +731,7 @@ int PCBNEW_JOBS_HANDLER::JobExportPos( JOB* aJob )
         PLACE_FILE_EXPORTER exporter( brd,
                                       aPosJob->m_units == JOB_EXPORT_PCB_POS::UNITS::MILLIMETERS,
                                       aPosJob->m_smdOnly, aPosJob->m_excludeFootprintsWithTh,
+                                      aPosJob->m_excludeDNP,
                                       frontSide, backSide,
                                       aPosJob->m_format == JOB_EXPORT_PCB_POS::FORMAT::CSV,
                                       aPosJob->m_useDrillPlaceFileOrigin,
@@ -643,7 +756,7 @@ int PCBNEW_JOBS_HANDLER::JobExportPos( JOB* aJob )
     return CLI::EXIT_CODES::OK;
 }
 
-extern FOOTPRINT* try_load_footprint( const wxFileName& aFileName, IO_MGR::PCB_FILE_T aFileType,
+extern FOOTPRINT* try_load_footprint( const wxFileName& aFileName, PCB_IO_MGR::PCB_FILE_T aFileType,
                                       const wxString& aName );
 
 
@@ -655,19 +768,20 @@ int PCBNEW_JOBS_HANDLER::JobExportFpUpgrade( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading footprint library\n" ) );
+        m_reporter->Report( _( "Loading footprint library\n" ), RPT_SEVERITY_INFO );
 
     if( !upgradeJob->m_outputLibraryPath.IsEmpty() )
     {
         if( wxFile::Exists( upgradeJob->m_outputLibraryPath ) ||
             wxDir::Exists( upgradeJob->m_outputLibraryPath) )
         {
-            wxFprintf( stderr, _( "Output path must not conflict with existing path\n" ) );
+            m_reporter->Report( _( "Output path must not conflict with existing path\n" ),
+                                RPT_SEVERITY_ERROR );
             return CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
         }
     }
 
-    PCB_PLUGIN  pcb_io( CTL_FOR_LIBRARY );
+    PCB_IO_KICAD_SEXPR  pcb_io( CTL_FOR_LIBRARY );
     FP_CACHE   fpLib( &pcb_io, upgradeJob->m_libraryPath );
 
     try
@@ -676,7 +790,7 @@ int PCBNEW_JOBS_HANDLER::JobExportFpUpgrade( JOB* aJob )
     }
     catch(...)
     {
-        wxFprintf( stderr, _( "Unable to load library\n" ) );
+        m_reporter->Report( _( "Unable to load library\n" ), RPT_SEVERITY_ERROR );
         return CLI::EXIT_CODES::ERR_UNKNOWN;
     }
 
@@ -692,7 +806,7 @@ int PCBNEW_JOBS_HANDLER::JobExportFpUpgrade( JOB* aJob )
 
     if( shouldSave )
     {
-        wxPrintf( _( "Saving footprint library\n" ) );
+        m_reporter->Report( _( "Saving footprint library\n" ), RPT_SEVERITY_INFO );
 
         try
         {
@@ -705,13 +819,13 @@ int PCBNEW_JOBS_HANDLER::JobExportFpUpgrade( JOB* aJob )
         }
         catch( ... )
         {
-            wxFprintf( stderr, _( "Unable to save library\n" ) );
+            m_reporter->Report( _( "Unable to save library\n" ), RPT_SEVERITY_ERROR );
             return CLI::EXIT_CODES::ERR_UNKNOWN;
         }
     }
     else
     {
-        wxPrintf( _( "Footprint library was not updated\n" ) );
+        m_reporter->Report( _( "Footprint library was not updated\n" ), RPT_SEVERITY_INFO );
     }
 
     return CLI::EXIT_CODES::OK;
@@ -726,9 +840,9 @@ int PCBNEW_JOBS_HANDLER::JobExportFpSvg( JOB* aJob )
         return CLI::EXIT_CODES::ERR_UNKNOWN;
 
     if( aJob->IsCli() )
-        wxPrintf( _( "Loading footprint library\n" ) );
+        m_reporter->Report( _( "Loading footprint library\n" ), RPT_SEVERITY_INFO );
 
-    PCB_PLUGIN pcb_io( CTL_FOR_LIBRARY );
+    PCB_IO_KICAD_SEXPR pcb_io( CTL_FOR_LIBRARY );
     FP_CACHE   fpLib( &pcb_io, svgJob->m_libraryPath );
 
     try
@@ -737,7 +851,7 @@ int PCBNEW_JOBS_HANDLER::JobExportFpSvg( JOB* aJob )
     }
     catch( ... )
     {
-        wxFprintf( stderr, _( "Unable to load library\n" ) );
+        m_reporter->Report( _( "Unable to load library\n" ), RPT_SEVERITY_ERROR );
         return CLI::EXIT_CODES::ERR_UNKNOWN;
     }
 
@@ -775,7 +889,10 @@ int PCBNEW_JOBS_HANDLER::JobExportFpSvg( JOB* aJob )
     }
 
     if( !svgJob->m_footprint.IsEmpty() && !singleFpPlotted )
-        wxFprintf( stderr, _( "The given footprint could not be found to export." ) );
+    {
+        m_reporter->Report( _( "The given footprint could not be found to export." ) + wxS( "\n" ),
+                            RPT_SEVERITY_ERROR );
+    }
 
     return CLI::EXIT_CODES::OK;
 }
@@ -787,6 +904,7 @@ int PCBNEW_JOBS_HANDLER::doFpExportSvg( JOB_FP_EXPORT_SVG* aSvgJob, const FOOTPR
     // until we refactor better plot api later
     std::unique_ptr<BOARD> brd;
     brd.reset( CreateEmptyBoard() );
+    brd->GetProject()->ApplyTextVars( aSvgJob->GetVarOverrides() );
 
     FOOTPRINT* fp = dynamic_cast<FOOTPRINT*>( aFootprint->Clone() );
 
@@ -811,10 +929,12 @@ int PCBNEW_JOBS_HANDLER::doFpExportSvg( JOB_FP_EXPORT_SVG* aSvgJob, const FOOTPR
     wxFileName outputFile;
     outputFile.SetPath( aSvgJob->m_outputDirectory );
     outputFile.SetName( aFootprint->GetFPID().GetLibItemName().wx_str() );
-    outputFile.SetExt( SVGFileExtension );
+    outputFile.SetExt( FILEEXT::SVGFileExtension );
 
-    wxPrintf( _( "Plotting footprint '%s' to '%s'\n" ),
-              aFootprint->GetFPID().GetLibItemName().wx_str(), outputFile.GetFullPath() );
+    m_reporter->Report( wxString::Format( _( "Plotting footprint '%s' to '%s'\n" ),
+                                          aFootprint->GetFPID().GetLibItemName().wx_str(),
+                                          outputFile.GetFullPath() ),
+                        RPT_SEVERITY_ACTION );
 
 
     PCB_PLOT_SVG_OPTIONS svgPlotOptions;
@@ -826,20 +946,310 @@ int PCBNEW_JOBS_HANDLER::doFpExportSvg( JOB_FP_EXPORT_SVG* aSvgJob, const FOOTPR
     svgPlotOptions.m_printMaskLayer = aSvgJob->m_printMaskLayer;
     svgPlotOptions.m_plotFrame = false;
 
-    if( !PCB_PLOT_SVG::Plot( brd.get(), svgPlotOptions ) )
-        wxFprintf( stderr, _( "Error creating svg file" ) );
+    if( !EXPORT_SVG::Plot( brd.get(), svgPlotOptions ) )
+        m_reporter->Report( _( "Error creating svg file" ) + wxS( "\n" ), RPT_SEVERITY_ERROR );
 
 
     return CLI::EXIT_CODES::OK;
 }
 
 
-REPORTER& PCBNEW_JOBS_HANDLER::Report( const wxString& aText, SEVERITY aSeverity )
+int PCBNEW_JOBS_HANDLER::JobExportDrc( JOB* aJob )
 {
-    if( aSeverity == RPT_SEVERITY_ERROR )
-        wxFprintf( stderr, wxS( "%s\n" ), aText );
-    else
-        wxPrintf( wxS( "%s\n" ), aText );
+    JOB_PCB_DRC* drcJob = dynamic_cast<JOB_PCB_DRC*>( aJob );
 
-    return *this;
+    if( drcJob == nullptr )
+        return CLI::EXIT_CODES::ERR_UNKNOWN;
+
+    if( aJob->IsCli() )
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
+
+    BOARD* brd = LoadBoard( drcJob->m_filename, true );
+    brd->GetProject()->ApplyTextVars( aJob->GetVarOverrides() );
+
+    if( drcJob->m_outputFile.IsEmpty() )
+    {
+        wxFileName fn = brd->GetFileName();
+        fn.SetName( fn.GetName() );
+
+        if( drcJob->m_format == JOB_PCB_DRC::OUTPUT_FORMAT::JSON )
+            fn.SetExt( FILEEXT::JsonFileExtension );
+        else
+            fn.SetExt( FILEEXT::ReportFileExtension );
+
+        drcJob->m_outputFile = fn.GetFullName();
+    }
+
+    EDA_UNITS units;
+    switch( drcJob->m_units )
+    {
+    case JOB_PCB_DRC::UNITS::INCHES:
+        units = EDA_UNITS::INCHES;
+        break;
+    case JOB_PCB_DRC::UNITS::MILS:
+        units = EDA_UNITS::MILS;
+        break;
+    case JOB_PCB_DRC::UNITS::MILLIMETERS:
+    default:
+        units = EDA_UNITS::MILLIMETRES; break;
+    }
+
+    std::shared_ptr<DRC_ENGINE> drcEngine = brd->GetDesignSettings().m_DRCEngine;
+    std::unique_ptr<NETLIST>    netlist = std::make_unique<NETLIST>();
+
+    drcEngine->SetDrawingSheet( getDrawingSheetProxyView( brd ) );
+
+    // BOARD_COMMIT uses TOOL_MANAGER to grab the board internally so we must give it one
+    TOOL_MANAGER* toolManager = new TOOL_MANAGER;
+    toolManager->SetEnvironment( brd, nullptr, nullptr, Kiface().KifaceSettings(), nullptr );
+
+    BOARD_COMMIT commit( toolManager );
+
+    m_reporter->Report( _( "Running DRC...\n" ), RPT_SEVERITY_INFO );
+
+    if( drcJob->m_parity )
+    {
+        typedef bool (*NETLIST_FN_PTR)( const wxString&, std::string& );
+
+        KIFACE*        eeschema = m_kiway->KiFACE( KIWAY::FACE_SCH );
+        wxFileName     schematicPath( drcJob->m_filename );
+        NETLIST_FN_PTR netlister = (NETLIST_FN_PTR) eeschema->IfaceOrAddress( KIFACE_NETLIST_SCHEMATIC );
+        std::string    netlist_str;
+
+        schematicPath.SetExt( FILEEXT::KiCadSchematicFileExtension );
+
+        if( !schematicPath.Exists() )
+            schematicPath.SetExt( FILEEXT::LegacySchematicFileExtension );
+
+        if( !schematicPath.Exists() )
+        {
+            m_reporter->Report( _( "Failed to find schematic for parity tests.\n" ),
+                                RPT_SEVERITY_INFO );
+        }
+        else
+        {
+            (*netlister)( schematicPath.GetFullPath(), netlist_str );
+
+            try
+            {
+                auto lineReader = new STRING_LINE_READER( netlist_str, _( "Eeschema netlist" ) );
+                KICAD_NETLIST_READER netlistReader( lineReader, netlist.get() );
+                netlistReader.LoadNetlist();
+            }
+            catch( const IO_ERROR& e )
+            {
+                m_reporter->Report( _( "Failed to fetch schematic netlist for parity tests.\n" ),
+                                    RPT_SEVERITY_INFO );
+            }
+
+            drcEngine->SetSchematicNetlist( netlist.get() );
+        }
+    }
+
+    drcEngine->SetProgressReporter( nullptr );
+    drcEngine->SetViolationHandler(
+            [&]( const std::shared_ptr<DRC_ITEM>& aItem, VECTOR2I aPos, int aLayer )
+            {
+                PCB_MARKER* marker = new PCB_MARKER( aItem, aPos, aLayer );
+                commit.Add( marker );
+            } );
+
+    brd->RecordDRCExclusions();
+    brd->DeleteMARKERs( true, true );
+    drcEngine->RunTests( units, drcJob->m_reportAllTrackErrors, drcJob->m_parity );
+    drcEngine->ClearViolationHandler();
+
+    commit.Push( _( "DRC" ), SKIP_UNDO | SKIP_SET_DIRTY );
+
+    // now "resolve" the drc exclusions again because its the only way to set exclusion status on
+    // a marker
+    for( PCB_MARKER* marker : brd->ResolveDRCExclusions( false ) )
+        brd->Add( marker );
+
+    std::shared_ptr<DRC_ITEMS_PROVIDER> markersProvider = std::make_shared<DRC_ITEMS_PROVIDER>(
+            brd, MARKER_BASE::MARKER_DRC, MARKER_BASE::MARKER_DRAWING_SHEET );
+
+    std::shared_ptr<DRC_ITEMS_PROVIDER> ratsnestProvider =
+            std::make_shared<DRC_ITEMS_PROVIDER>( brd, MARKER_BASE::MARKER_RATSNEST );
+
+    std::shared_ptr<DRC_ITEMS_PROVIDER> fpWarningsProvider =
+            std::make_shared<DRC_ITEMS_PROVIDER>( brd, MARKER_BASE::MARKER_PARITY );
+
+    markersProvider->SetSeverities( drcJob->m_severity );
+    ratsnestProvider->SetSeverities( drcJob->m_severity );
+    fpWarningsProvider->SetSeverities( drcJob->m_severity );
+
+    m_reporter->Report( wxString::Format( _( "Found %d violations\n" ),
+                                          markersProvider->GetCount() ),
+                        RPT_SEVERITY_INFO );
+    m_reporter->Report( wxString::Format( _( "Found %d unconnected items\n" ),
+                                          ratsnestProvider->GetCount() ),
+                        RPT_SEVERITY_INFO );
+
+    if( drcJob->m_parity )
+    {
+        m_reporter->Report( wxString::Format( _( "Found %d schematic parity issues\n" ),
+                                              fpWarningsProvider->GetCount() ),
+                            RPT_SEVERITY_INFO );
+    }
+
+    DRC_REPORT reportWriter( brd, units, markersProvider, ratsnestProvider, fpWarningsProvider );
+
+    bool wroteReport = false;
+    if( drcJob->m_format == JOB_PCB_DRC::OUTPUT_FORMAT::JSON )
+        wroteReport = reportWriter.WriteJsonReport( drcJob->m_outputFile );
+    else
+        wroteReport = reportWriter.WriteTextReport( drcJob->m_outputFile );
+
+    if( !wroteReport )
+    {
+        m_reporter->Report( wxString::Format( _( "Unable to save DRC report to %s\n" ), drcJob->m_outputFile ),
+                            RPT_SEVERITY_INFO );
+        return CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
+    }
+
+    m_reporter->Report( wxString::Format( _( "Saved DRC Report to %s\n" ), drcJob->m_outputFile ),
+                        RPT_SEVERITY_INFO );
+
+    if( drcJob->m_exitCodeViolations )
+    {
+        if( markersProvider->GetCount() > 0 || ratsnestProvider->GetCount() > 0
+            || fpWarningsProvider->GetCount() > 0 )
+        {
+            return CLI::EXIT_CODES::ERR_RC_VIOLATIONS;
+        }
+    }
+
+    return CLI::EXIT_CODES::SUCCESS;
 }
+
+
+int PCBNEW_JOBS_HANDLER::JobExportIpc2581( JOB* aJob )
+{
+    JOB_EXPORT_PCB_IPC2581* job = dynamic_cast<JOB_EXPORT_PCB_IPC2581*>( aJob );
+
+    if( job == nullptr )
+        return CLI::EXIT_CODES::ERR_UNKNOWN;
+
+    if( job->IsCli() )
+        m_reporter->Report( _( "Loading board\n" ), RPT_SEVERITY_INFO );
+
+    BOARD* brd = LoadBoard( job->m_filename, true );
+
+    if( job->m_outputFile.IsEmpty() )
+    {
+        wxFileName fn = brd->GetFileName();
+        fn.SetName( fn.GetName() );
+        fn.SetExt( FILEEXT::Ipc2581FileExtension );
+
+        job->m_outputFile = fn.GetFullName();
+    }
+
+    STRING_UTF8_MAP props;
+    props["units"] =
+            job->m_units == JOB_EXPORT_PCB_IPC2581::IPC2581_UNITS::MILLIMETERS ? "mm" : "inch";
+    props["sigfig"] = wxString::Format( "%d", job->m_units );
+    props["version"] = job->m_version == JOB_EXPORT_PCB_IPC2581::IPC2581_VERSION::C ? "C" : "B";
+    props["OEMRef"] = job->m_colInternalId;
+    props["mpn"] = job->m_colMfgPn;
+    props["mfg"] = job->m_colMfg;
+    props["dist"] = job->m_colDist;
+    props["distpn"] = job->m_colDistPn;
+
+    wxString tempFile = wxFileName::CreateTempFileName( wxS( "pcbnew_ipc" ) );
+    try
+    {
+        IO_RELEASER<PCB_IO> pi( PCB_IO_MGR::PluginFind( PCB_IO_MGR::IPC2581 ) );
+        pi->SetProgressReporter( m_progressReporter );
+        pi->SaveBoard( tempFile, brd, &props );
+    }
+    catch( const IO_ERROR& ioe )
+    {
+        m_reporter->Report( wxString::Format( _( "Error generating IPC2581 file '%s'.\n%s" ),
+                                              job->m_filename, ioe.What() ),
+                            RPT_SEVERITY_ERROR );
+
+        wxRemoveFile( tempFile );
+
+        return CLI::EXIT_CODES::ERR_UNKNOWN;
+    }
+
+    if( job->m_compress )
+    {
+        wxFileName tempfn = job->m_outputFile;
+        tempfn.SetExt( FILEEXT::Ipc2581FileExtension );
+        wxFileName zipfn = tempFile;
+        zipfn.SetExt( "zip" );
+
+        wxFFileOutputStream fnout( zipfn.GetFullPath() );
+        wxZipOutputStream   zip( fnout );
+        wxFFileInputStream  fnin( tempFile );
+
+        zip.PutNextEntry( tempfn.GetFullName() );
+        fnin.Read( zip );
+        zip.Close();
+        fnout.Close();
+
+        wxRemoveFile( tempFile );
+        tempFile = zipfn.GetFullPath();
+    }
+
+    // If save succeeded, replace the original with what we just wrote
+    if( !wxRenameFile( tempFile, job->m_outputFile ) )
+    {
+        m_reporter->Report( wxString::Format( _( "Error generating IPC2581 file '%s'.\n"
+                                                 "Failed to rename temporary file '%s." )
+                                                      + wxS( "\n" ),
+                                              job->m_outputFile, tempFile ),
+                            RPT_SEVERITY_ERROR );
+    }
+
+    return CLI::EXIT_CODES::SUCCESS;
+}
+
+
+DS_PROXY_VIEW_ITEM* PCBNEW_JOBS_HANDLER::getDrawingSheetProxyView( BOARD* aBrd )
+{
+    DS_PROXY_VIEW_ITEM* drawingSheet = new DS_PROXY_VIEW_ITEM( pcbIUScale,
+                                                               &aBrd->GetPageSettings(),
+                                                               aBrd->GetProject(),
+                                                               &aBrd->GetTitleBlock(),
+                                                               &aBrd->GetProperties() );
+
+    drawingSheet->SetSheetName( std::string() );
+    drawingSheet->SetSheetPath( std::string() );
+    drawingSheet->SetIsFirstPage( true );
+
+    drawingSheet->SetFileName( TO_UTF8( aBrd->GetFileName() ) );
+
+    return drawingSheet;
+}
+
+
+void PCBNEW_JOBS_HANDLER::loadOverrideDrawingSheet( BOARD* aBrd, const wxString& aSheetPath )
+{
+    // dont bother attempting to load a empty path, if there was one
+    if( aSheetPath.IsEmpty() )
+        return;
+
+    auto loadSheet = [&]( const wxString& path ) -> bool
+    {
+        BASE_SCREEN::m_DrawingSheetFileName = path;
+        wxString filename = DS_DATA_MODEL::ResolvePath( BASE_SCREEN::m_DrawingSheetFileName,
+                                                        aBrd->GetProject()->GetProjectPath() );
+        if( !DS_DATA_MODEL::GetTheInstance().LoadDrawingSheet( filename ) )
+        {
+            m_reporter->Report( wxString::Format( _( "Error loading drawing sheet '%s'." )+ wxS( "\n" ), path ),
+                                RPT_SEVERITY_ERROR );
+            return false;
+        }
+
+        return true;
+    };
+
+    if( loadSheet( aSheetPath ) )
+        return;
+
+    // failed loading custom path, revert back to default
+    loadSheet( aBrd->GetProject()->GetProjectFile().m_BoardDrawingSheetFile );
+}
\ No newline at end of file
