@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013-2017 CERN
- * Copyright (C) 2017-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2017-2023 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
@@ -24,43 +24,29 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include <advanced_config.h>
+#include <functional>
 #include <limits>
+#include <kiplatform/ui.h>
 #include <board.h>
-#include <board_design_settings.h>
-#include <footprint.h>
-#include <fp_shape.h>
-#include <collectors.h>
+#include <board_commit.h>
+#include <gal/graphics_abstraction_layer.h>
+#include <pad.h>
+#include <pcb_group.h>
+#include <pcb_generator.h>
 #include <pcb_edit_frame.h>
-#include <drawing_sheet/ds_proxy_view_item.h>
-#include <kiway.h>
-#include <pcbnew_settings.h>
 #include <spread_footprints.h>
 #include <tools/pcb_actions.h>
 #include <tools/pcb_selection_tool.h>
 #include <tools/edit_tool.h>
 #include <tools/pcb_grid_helper.h>
 #include <tools/drc_tool.h>
-#include <tools/drawing_tool.h>
 #include <tools/zone_filler_tool.h>
-#include <view/view_controls.h>
-#include <connectivity/connectivity_algo.h>
-#include <connectivity/connectivity_items.h>
-#include <cassert>
-#include <functional>
-#include <wx/hyperlink.h>
 #include <router/router_tool.h>
 #include <dialogs/dialog_move_exact.h>
-#include <dialogs/dialog_unit_entry.h>
-#include <board_commit.h>
-#include <pcb_group.h>
-#include <pcb_target.h>
 #include <zone_filler.h>
 #include <drc/drc_engine.h>
 #include <drc/drc_item.h>
 #include <drc/drc_rule.h>
-#include <pad.h>
-#include <geometry/shape_segment.h>
 #include <drc/drc_interactive_courtyard_clearance.h>
 
 
@@ -84,12 +70,8 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
                 {
                     BOARD_ITEM* item = aCollector[i];
 
-                    switch( item->Type() )
-                    {
-                    case PCB_TRACE_T: aCollector.Remove( item ); break;
-
-                    default: break;
-                    }
+                    if( item->Type() == PCB_TRACE_T )
+                        aCollector.Remove( item );
                 }
             },
             true /* prompt user regarding locked items */ );
@@ -97,42 +79,27 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
     if( selection.Size() < 2 )
         return 0;
 
+    BOARD_COMMIT  localCommit( this );
+    BOARD_COMMIT* commit = dynamic_cast<BOARD_COMMIT*>( aEvent.Commit() );
+
+    if( !commit )
+        commit = &localCommit;
+
     std::vector<EDA_ITEM*> sorted = selection.GetItemsSortedBySelectionOrder();
 
-    // When editing footprints, all items have the same parent
-    if( IsFootprintEditor() )
+    // Save items, so changes can be undone
+    for( EDA_ITEM* item : selection )
     {
-        m_commit->Modify( selection.Front() );
-    }
-    else
-    {
-        // Save items, so changes can be undone
-        for( EDA_ITEM* item : selection )
-        {
-            // Don't double move footprint pads, fields, etc.
-            //
-            // For PCB_GROUP_T, the parent is the board.
-            if( item->GetParent() && item->GetParent()->IsSelected() )
-                continue;
-
-            m_commit->Modify( item );
-
-            // If moving a group, record position of all the descendants for undo
-            if( item->Type() == PCB_GROUP_T )
-            {
-                PCB_GROUP* group = static_cast<PCB_GROUP*>( item );
-                group->RunOnDescendants( [&]( BOARD_ITEM* bItem )
-                                         {
-                                             m_commit->Modify( bItem );
-                                         });
-            }
-        }
+        if( !item->IsNew() && !item->IsMoving() )
+            commit->Modify( item );
     }
 
     for( size_t i = 0; i < sorted.size() - 1; i++ )
     {
-        BOARD_ITEM* a = static_cast<BOARD_ITEM*>( sorted[i] );
-        BOARD_ITEM* b = static_cast<BOARD_ITEM*>( sorted[( i + 1 ) % sorted.size()] );
+        BOARD_ITEM* a = dynamic_cast<BOARD_ITEM*>( sorted[i] );
+        BOARD_ITEM* b = dynamic_cast<BOARD_ITEM*>( sorted[( i + 1 ) % sorted.size()] );
+
+        wxCHECK2( a && b, continue );
 
         // Swap X,Y position
         VECTOR2I aPos = a->GetPosition(), bPos = b->GetPosition();
@@ -140,13 +107,6 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
         a->SetPosition( aPos );
         b->SetPosition( bPos );
 
-        // Pads need special handling to keep their offset from their parent
-        if( a->Type() == PCB_PAD_T )
-            static_cast<PAD*>( a )->SetLocalCoord();
-
-        if( b->Type() == PCB_PAD_T )
-            static_cast<PAD*>( b )->SetLocalCoord();
-
         // Handle footprints specially. They can be flipped to the back of the board which
         // requires a special transformation.
         if( a->Type() == PCB_FOOTPRINT_T && b->Type() == PCB_FOOTPRINT_T )
@@ -182,8 +142,8 @@ int EDIT_TOOL::Swap( const TOOL_EVENT& aEvent )
         }
     }
 
-    if( !m_dragging )
-        m_commit->Push( _( "Swap" ) );
+    if( !localCommit.Empty() )
+        localCommit.Push( _( "Swap" ) );
 
     m_toolMgr->ProcessEvent( EVENTS::SelectedItemsModified );
 
@@ -199,6 +159,7 @@ int EDIT_TOOL::PackAndMoveFootprints( const TOOL_EVENT& aEvent )
         return 0;
     }
 
+    BOARD_COMMIT   commit( this );
     PCB_SELECTION& selection = m_selectionTool->RequestSelection(
             []( const VECTOR2I& aPt, GENERAL_COLLECTOR& aCollector, PCB_SELECTION_TOOL* sTool )
             {
@@ -229,25 +190,56 @@ int EDIT_TOOL::PackAndMoveFootprints( const TOOL_EVENT& aEvent )
 
     for( FOOTPRINT* item : footprintsToPack )
     {
-        m_commit->Modify( item );
+        commit.Modify( item );
+        item->SetFlags( IS_MOVING );
         footprintsBbox.Merge( item->GetBoundingBox( false, false ) );
     }
 
     SpreadFootprints( &footprintsToPack, footprintsBbox.Normalize().GetOrigin(), false );
 
-    return doMoveSelection( aEvent, _( "Pack footprints" ) );
+    if( doMoveSelection( aEvent, &commit ) )
+        commit.Push( _( "Pack footprints" ) );
+    else
+        commit.Revert();
+
+    return 0;
 }
 
 
 int EDIT_TOOL::Move( const TOOL_EVENT& aEvent )
 {
-    if( isRouterActive() )
+    if( isRouterActive() || m_dragging )
     {
         wxBell();
         return 0;
     }
 
-    return doMoveSelection( aEvent, _( "Move" ) );
+    if( BOARD_COMMIT* commit = dynamic_cast<BOARD_COMMIT*>( aEvent.Commit() ) )
+    {
+        wxCHECK( aEvent.SynchronousState(), 0 );
+        aEvent.SynchronousState()->store( STS_RUNNING );
+
+        if( doMoveSelection( aEvent, commit ) )
+            aEvent.SynchronousState()->store( STS_FINISHED );
+        else
+            aEvent.SynchronousState()->store( STS_CANCELLED );
+    }
+    else
+    {
+        BOARD_COMMIT localCommit( this );
+
+        if( doMoveSelection( aEvent, &localCommit ) )
+            localCommit.Push( _( "Move" ) );
+        else
+            localCommit.Revert();
+    }
+
+    // Notify point editor.  (While doMoveSelection() will re-select the items and post this
+    // event, it's done before the edit flags are cleared in BOARD_COMMIT::Push() so the point
+    // editor doesn't fire up.)
+    m_toolMgr->ProcessEvent( EVENTS::SelectedEvent );
+
+    return 0;
 }
 
 
@@ -285,7 +277,7 @@ VECTOR2I EDIT_TOOL::getSafeMovement( const VECTOR2I& aMovement, const BOX2I& aSo
 }
 
 
-int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommitMessage )
+bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit )
 {
     bool moveWithReference = aEvent.IsAction( &PCB_ACTIONS::moveWithReference );
     bool moveIndividually = aEvent.IsAction( &PCB_ACTIONS::moveIndividually );
@@ -299,8 +291,6 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
     wxString              status;
     size_t                itemIdx = 0;
 
-    editFrame->PushTool( aEvent );
-
     // Be sure that there is at least one item that we can modify. If nothing was selected before,
     // try looking for the stuff under mouse cursor (i.e. KiCad old-style hover selection)
     PCB_SELECTION& selection = m_selectionTool->RequestSelection(
@@ -308,43 +298,16 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             {
                 sTool->FilterCollectorForMarkers( aCollector );
                 sTool->FilterCollectorForHierarchy( aCollector, true );
+                sTool->FilterCollectorForFreePads( aCollector );
             },
-            // Prompt user regarding locked items if in board editor and in free-pad-mode (if
-            // we're not in free-pad mode we delay this until the second RequestSelection()).
-            !m_isFootprintEditor && cfg->m_AllowFreePads );
+            true /* prompt user regarding locked items */ );
 
     if( m_dragging || selection.Empty() )
-    {
-        editFrame->PopTool( aEvent );
-        return 0;
-    }
-
-    LSET     item_layers = selection.GetSelectionLayers();
-    bool     is_hover    = selection.IsHover(); // N.B. This must be saved before the second call
-                                                // to RequestSelection() below
-    VECTOR2I pickedReferencePoint;
-
-    // Now filter out pads if not in free pads mode.  We cannot do this in the first
-    // RequestSelection() as we need the item_layers when a pad is the selection front.
-    if( !m_isFootprintEditor && !cfg->m_AllowFreePads )
-    {
-        selection = m_selectionTool->RequestSelection(
-                []( const VECTOR2I& aPt, GENERAL_COLLECTOR& aCollector, PCB_SELECTION_TOOL* sTool )
-                {
-                    sTool->FilterCollectorForMarkers( aCollector );
-                    sTool->FilterCollectorForHierarchy( aCollector, true );
-                    sTool->FilterCollectorForFreePads( aCollector );
-                },
-                true /* prompt user regarding locked items */ );
-    }
-
-    if( selection.Empty() )
-    {
-        editFrame->PopTool( aEvent );
-        return 0;
-    }
+        return false;
 
+    editFrame->PushTool( aEvent );
     Activate();
+
     // Must be done after Activate() so that it gets set into the correct context
     controls->ShowCursor( true );
     controls->SetAutoPan( true );
@@ -354,7 +317,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             [editFrame]( bool constrained )
             {
                 editFrame->DisplayConstraintsMsg( constrained ? _( "Constrain to H, V, 45" )
-                                                              : wxT( "" ) );
+                                                              : wxString( wxT( "" ) ) );
             };
 
     auto updateStatusPopup =
@@ -372,7 +335,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 else if( item->Type() == PCB_PAD_T )
                 {
                     PAD*       pad = static_cast<PAD*>( item );
-                    FOOTPRINT* fp = static_cast<FOOTPRINT*>( pad->GetParentFootprint() );
+                    FOOTPRINT* fp = pad->GetParentFootprint();
                     msg = wxString::Format( _( "%s pad %s" ), fp->GetReference(), pad->GetNumber() );
                 }
                 else
@@ -388,18 +351,15 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
     for( EDA_ITEM* item : selection )
     {
-        BOARD_ITEM* boardItem = dynamic_cast<BOARD_ITEM*>( item );
-        FOOTPRINT*  footprint = dynamic_cast<FOOTPRINT*>( item );
-
-        if( boardItem )
+        if( BOARD_ITEM* boardItem = dynamic_cast<BOARD_ITEM*>( item ) )
         {
-            if( !is_hover )
+            if( !selection.IsHover() )
                 orig_items.push_back( boardItem );
 
             sel_items.push_back( boardItem );
         }
 
-        if( footprint )
+        if( FOOTPRINT* footprint = dynamic_cast<FOOTPRINT*>( item ) )
         {
             for( PAD* pad : footprint->Pads() )
                 sel_items.push_back( pad );
@@ -410,14 +370,16 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
         }
     }
 
+    VECTOR2I pickedReferencePoint;
+
     if( moveWithReference && !pickReferencePoint( _( "Select reference point for move..." ), "", "",
                                                   pickedReferencePoint ) )
     {
-        if( is_hover )
-            m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+        if( selection.IsHover() )
+            m_toolMgr->RunAction( PCB_ACTIONS::selectionClear );
 
         editFrame->PopTool( aEvent );
-        return 0;
+        return false;
     }
 
     if( moveIndividually )
@@ -425,11 +387,14 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
         orig_items.clear();
 
         for( EDA_ITEM* item : selection.GetItemsSortedBySelectionOrder() )
-            orig_items.push_back( static_cast<BOARD_ITEM*>( item ) );
+        {
+            if( BOARD_ITEM* boardItem = dynamic_cast<BOARD_ITEM*>( item ) )
+                orig_items.push_back( boardItem );
+        }
 
         updateStatusPopup( orig_items[ itemIdx ], itemIdx + 1, orig_items.size() );
         statusPopup.Popup();
-        statusPopup.Move( wxGetMousePosition() + wxPoint( 20, 20 ) );
+        statusPopup.Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
         canvas()->SetStatusPopup( statusPopup.GetPanel() );
 
         m_selectionTool->ClearSelection();
@@ -445,14 +410,15 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
     VECTOR2D        bboxMovement;
     BOX2I           originalBBox;
     bool            updateBBox = true;
+    LSET            layers( editFrame->GetActiveLayer() );
     PCB_GRID_HELPER grid( m_toolMgr, editFrame->GetMagneticItemsSettings() );
     TOOL_EVENT      copy = aEvent;
     TOOL_EVENT*     evt = &copy;
     VECTOR2I        prevPos;
+    bool            enableLocalRatsnest = true;
 
     bool hv45Mode        = false;
     bool eatFirstMouseUp = true;
-    bool hasRedrawn3D    = false;
     bool allowRedraw3D   = cfg->m_Display.m_Live3DRefresh;
     bool showCourtyardConflicts = !m_isFootprintEditor && cfg->m_ShowCourtyardCollisions;
 
@@ -469,7 +435,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
     displayConstraintsMessage( hv45Mode );
 
     // Prime the pump
-    m_toolMgr->RunAction( ACTIONS::refreshPreview );
+    m_toolMgr->PostAction( ACTIONS::refreshPreview );
 
     // Main loop: keep receiving events
     do
@@ -495,7 +461,8 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
                 VECTOR2I mousePos( controls->GetMousePosition() );
 
-                m_cursor = grid.BestSnapAnchor( mousePos, item_layers, sel_items );
+                m_cursor = grid.BestSnapAnchor( mousePos, layers,
+                                                grid.GetSelectionGrid( selection ), sel_items );
 
                 if( controls->GetSettings().m_lastKeyboardCursorPositionValid )
                 {
@@ -552,22 +519,22 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 // Drag items to the current cursor position
                 for( EDA_ITEM* item : sel_items )
                 {
-                    // Don't double move footprint pads, fields, etc.
-                    //
-                    // For PCB_GROUP_T, we make sure the selection includes only the top level
-                    // group and not its descendants.
+                    // Don't double move child items.
                     if( !item->GetParent() || !item->GetParent()->IsSelected() )
                         static_cast<BOARD_ITEM*>( item )->Move( movement );
 
-                    if( !redraw3D && item->Type() == PCB_FOOTPRINT_T )
+                    if( item->Type() == PCB_GENERATOR_T && sel_items.size() == 1 )
+                    {
+                        m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genUpdateEdit, aCommit,
+                                                         static_cast<PCB_GENERATOR*>( item ) );
+                    }
+
+                    if( item->Type() == PCB_FOOTPRINT_T )
                         redraw3D = true;
                 }
 
                 if( redraw3D && allowRedraw3D )
-                {
                     editFrame->Update3DView( false, true );
-                    hasRedrawn3D = true;
-                }
 
                 if( showCourtyardConflicts && drc_on_move->m_FpInMove.size() )
                 {
@@ -584,37 +551,35 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
 
                 m_dragging = true;
 
-                // When editing footprints, all items have the same parent
-                if( IsFootprintEditor() )
+                for( EDA_ITEM* item : selection )
                 {
-                    m_commit->Modify( selection.Front() );
-                }
-                else
-                {
-                    // Save items, so changes can be undone
-                    for( EDA_ITEM* item : selection )
+                    if( item->GetParent() && item->GetParent()->IsSelected() )
+                        continue;
+
+                    if( !item->IsNew() && !item->IsMoving() )
                     {
-                        // Don't double move footprint pads, fields, etc.
-                        //
-                        // For PCB_GROUP_T, the parent is the board.
-                        if( item->GetParent() && item->GetParent()->IsSelected() )
-                            continue;
-
-                        m_commit->Modify( item );
-
-                        // If moving a group, record position of all the descendants for undo
-                        if( item->Type() == PCB_GROUP_T )
+                        if( item->Type() == PCB_GENERATOR_T && sel_items.size() == 1 )
                         {
-                            PCB_GROUP* group = static_cast<PCB_GROUP*>( item );
-                            group->RunOnDescendants( [&]( BOARD_ITEM* bItem )
-                                                     {
-                                                         m_commit->Modify( bItem );
-                                                     });
+                            enableLocalRatsnest = false;
+
+                            m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genStartEdit, aCommit,
+                                                             static_cast<PCB_GENERATOR*>( item ) );
                         }
+                        else
+                        {
+                            aCommit->Modify( item );
+                        }
+
+                        item->SetFlags( IS_MOVING );
+
+                        static_cast<BOARD_ITEM*>( item )->RunOnDescendants(
+                                [&]( BOARD_ITEM* bItem )
+                                {
+                                    item->SetFlags( IS_MOVING );
+                                } );
                     }
                 }
 
-                editFrame->UndoRedoBlock( true );
                 m_cursor = controls->GetCursorPosition();
 
                 if( selection.HasReferencePoint() )
@@ -644,13 +609,26 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 }
                 else
                 {
-                    for( BOARD_ITEM* item : sel_items )
+                    if( showCourtyardConflicts )
                     {
-                        if( showCourtyardConflicts && item->Type() == PCB_FOOTPRINT_T )
-                            drc_on_move->m_FpInMove.push_back( static_cast<FOOTPRINT*>( item ) );
+                        std::vector<FOOTPRINT*>& FPs = drc_on_move->m_FpInMove;
+
+                        for( BOARD_ITEM* item : sel_items )
+                        {
+                            if( item->Type() == PCB_FOOTPRINT_T )
+                                FPs.push_back( static_cast<FOOTPRINT*>( item ) );
+
+                            item->RunOnDescendants(
+                                    [&]( BOARD_ITEM* descendent )
+                                    {
+                                        if( descendent->Type() == PCB_FOOTPRINT_T )
+                                            FPs.push_back( static_cast<FOOTPRINT*>( descendent ) );
+                                    } );
+                        }
                     }
 
                     m_cursor = grid.BestDragOrigin( originalCursorPos, sel_items,
+                                                    grid.GetSelectionGrid( selection ),
                                                     &m_selectionTool->GetFilter() );
 
                     // Set the current cursor position to the first dragged item origin, so the
@@ -684,9 +662,10 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
             }
 
-            statusPopup.Move( wxGetMousePosition() + wxPoint( 20, 20 ) );
+            statusPopup.Move( KIPLATFORM::UI::GetMousePosition() + wxPoint( 20, 20 ) );
 
-            m_toolMgr->RunAction( PCB_ACTIONS::updateLocalRatsnest, false, new VECTOR2I( movement ) );
+            if( enableLocalRatsnest )
+                m_toolMgr->PostAction( PCB_ACTIONS::updateLocalRatsnest, movement );
         }
         else if( evt->IsCancelInteractive() || evt->IsActivate() )
         {
@@ -696,21 +675,13 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
             restore_state = true; // Canceling the tool means that items have to be restored
             break;                // Finish
         }
-        else if( evt->IsAction( &ACTIONS::undo ) )
+        else if( evt->IsAction( &ACTIONS::undo ) || evt->IsAction( &ACTIONS::doDelete ) )
         {
             restore_state = true; // Perform undo locally
             break;                // Finish
         }
-        else if( evt->IsAction( &ACTIONS::doDelete ) || evt->IsAction( &ACTIONS::cut ) )
+        else if( evt->IsAction( &ACTIONS::duplicate ) || evt->IsAction( &ACTIONS::cut ) )
         {
-            // Dispatch TOOL_ACTIONs
-            evt->SetPassEvent();
-            break; // finish -- there is no further processing for removed items
-        }
-        else if( evt->IsAction( &ACTIONS::duplicate ) )
-        {
-            evt->SetPassEvent();
-            break; // finish -- Duplicate tool will start a new Move with the dup'ed items
         }
         else if( evt->IsAction( &PCB_ACTIONS::rotateCw )
                 || evt->IsAction( &PCB_ACTIONS::rotateCcw )
@@ -725,7 +696,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
         else if( evt->IsMouseUp( BUT_LEFT ) || evt->IsClick( BUT_LEFT ) || isSkip )
         {
             // Eat mouse-up/-click events that leaked through from the lock dialog
-            if( eatFirstMouseUp && evt->Parameter<intptr_t>() != ACTIONS::CURSOR_CLICK )
+            if( eatFirstMouseUp && !evt->IsAction( &ACTIONS::cursorClick ) )
             {
                 eatFirstMouseUp = false;
                 continue;
@@ -736,6 +707,7 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                 if( isSkip )
                     orig_items[itemIdx]->SetPosition( originalPos );
 
+                view()->Update( orig_items[itemIdx] );
                 rebuildConnectivity();
 
                 if( ++itemIdx < orig_items.size() )
@@ -753,8 +725,8 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
                     updateStatusPopup( nextItem, itemIdx + 1, orig_items.size() );
 
                     // Pick up new item
-                    m_commit->Modify( nextItem );
-                    nextItem->SetPosition( controls->GetMousePosition( true ) );
+                    aCommit->Modify( nextItem );
+                    nextItem->Move( controls->GetCursorPosition( true ) - nextItem->GetPosition() );
 
                     continue;
                 }
@@ -801,45 +773,40 @@ int EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, const wxString& aCommi
     controls->SetAutoPan( false );
 
     m_dragging = false;
-    editFrame->UndoRedoBlock( false );
 
     // Discard reference point when selection is "dropped" onto the board
     selection.ClearReferencePoint();
 
     // Unselect all items to clear selection flags and then re-select the originally selected
-    // items (after the potential Revert()).
-    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+    // items.
+    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear );
 
-    // TODO: there's an encapsulation leak here: this commit often has more than just the move
-    // in it; for instance it might have a paste, append board, etc. as well.
     if( restore_state )
     {
-        m_commit->Revert();
-        m_selectionTool->RebuildSelection();
-
-        // Mainly for point editor, but there might be other clients that need to adjust to
-        // reverted state.
-        m_toolMgr->PostEvent( EVENTS::SelectedItemsMoved );
-
-        // Property panel needs to know about the reselect
-        m_toolMgr->PostEvent( EVENTS::SelectedItemsModified );
-
-        if( hasRedrawn3D )
-            editFrame->Update3DView( false, true );
+        if( sel_items.size() == 1 && sel_items.back()->Type() == PCB_GENERATOR_T )
+        {
+            m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genRevertEdit, aCommit,
+                                             static_cast<PCB_GENERATOR*>( sel_items.back() ) );
+        }
     }
     else
     {
-        m_commit->Push( aCommitMessage );
-        m_toolMgr->RunAction( PCB_ACTIONS::selectItems, true, &orig_items );
+        if( sel_items.size() == 1 && sel_items.back()->Type() == PCB_GENERATOR_T )
+        {
+            m_toolMgr->RunSynchronousAction( PCB_ACTIONS::genPushEdit, aCommit,
+                                             static_cast<PCB_GENERATOR*>( sel_items.back() ) );
+        }
+
+        EDA_ITEMS oItems( orig_items.begin(), orig_items.end() );
+        m_toolMgr->RunAction<EDA_ITEMS*>( PCB_ACTIONS::selectItems, &oItems );
     }
 
-    m_toolMgr->GetTool<DRAWING_TOOL>()->UpdateStatusBar();
     // Remove the dynamic ratsnest from the screen
-    m_toolMgr->RunAction( PCB_ACTIONS::hideLocalRatsnest, true );
+    m_toolMgr->RunAction( PCB_ACTIONS::hideLocalRatsnest );
 
     editFrame->PopTool( aEvent );
     editFrame->GetCanvas()->SetCurrentCursor( KICURSOR::ARROW );
 
-    return restore_state ? -1 : 0;
+    return !restore_state;
 }
 
